//
// Validation Framework, built with jef and jQuery 
//
(function(jef, $){
		
	/********************************************************************/
	/**************    Valid Namespace Methods             **************/
	/********************************************************************/
		
	if(jef.valid){
		return;
	}
	var valid = jef.valid = {};	
	
	//
	// Settings Method
	//
	
	/**
	* Multiple Method for Settings.
	GET:
	* @param {string|Array} key The settings key. Must be a validate object path (@see jef.navigate)
	* @return The found value
	SET through an object path:
	* @param {string} key The settings key. Must be a validate object path (@see jef.navigate)
	* @param {anything} value The value te be setted
	SET directly on settings object:
	* @param {object} settings The new settings to be setted on to current settings
	*/
	valid.settings = function(key, value){
		var isString = jef.isString(key);
		var isArray = jef.isArray(key);
		var oldValue, stack, dstObject = settings;
		if((isString || isArray) && value === undefined){
			// Get
			return jef.navigate(settings, key);
		} else if(isString || isArray){
			// Set through path
			if(isString)
				key = key.split('.');
			stack = [];
			oldValue = jef.navigate(settings, key, stack);
			if(stack.length > 1){
				dstObject = stack[stack.length - 2];
			}
			setSettingValue_(dstObject, key[stack.length - 1], value, oldValue);
		} else if(jef.isSimpleObject(key)) {
			// Direct Set
			for(var name in key){
				setSettingValue_(settings, name, key[name], settings[name]);
			}
		}
	};
			
	// Default settings		
	var settings = {	
		/**
		* Format field when initializing validation
		*/
		formatOnInitValidation : true,
		/**
		* Event types for witch format will be apply during validation
		*/
		applyFormatEventTypes : ['blur','submit'],
		/**
		 * Default parser for number (must be a function)
		 */
		defaultNumberParser : jef.number.parseAtAllCost,
		/**
		* Default initialized field events if no one is defined in validation options.
		* Attributes correspond to type of fields [get by $(selector).attr('type')]
		*/ 
		defaultFieldEvents : {
			'radio' : ['blur', 'click'],
			'checkbox' : ['blur', 'click'],
			'select-multiple' : ['blur', 'change'],
			'select-one' : ['blur', 'change']
		},
		/**
		 * Parse form on DOM ready
		 */
		parseFormsOnDomReady : true,
		/**
		 * Parse form on DOM ready
		 */
		parseFieldsOnDomReady : true,
		/**
		* Init field validation on initializaing form validation 
		*/
		initFieldsValidationWithForm : true,
		/**
		* Stop field validation on stopping form validation 
		*/
		stopFieldsValidationWithForm : true,
		/**
		* Fix (re-bound) field validation on validating form
		*/
		fixFieldsValidationWithForm : false,
		/**	
		* Stop form validation on first error
		*/ 
		stopFormValidationOnFirstError : false,
		/**
		 * Don't execute validations until form will be submit
		 */
		skipValidationBeforeSubmit : false,
		/**
		* Char codes that will be skipped in key block validation witch mean 
		* that they won't stop event propagation. Typically, they are 
		* control keys as tab, remove, enter, up, left, etc...
		* On opera, control keys aren't bind to charCode 0 !
		*/		
		keyBlockSkippedCharCode : jQuery.browser.opera ? [0,8,9,13,27,33,34,35,36,37,38,39,40,45,46] : [0,8,13],
		/**
		 * Default style (css) for empty text feature
		 */
		defaultEmptyTextStyle : { 'font-style': 'italic', 'color': '#7f7f7f' },
		/**
		 * Overwrite jquery datepicker format and parse methods
		 */
		overwriteDatepickerFormat : true
	};
	
	var defaultFieldEvents = ['blur'];	
	var getDefaultFieldEvents_ = function(we){
		return settings.defaultFieldEvents[we.attr('type')] || defaultFieldEvents;
	};	
	var setSettingValue_ = function(dst, key, value, oldValue){
		if(jef.isSimpleObject(oldValue)){
			dst[key] = jef.extend(oldValue, value);
		} else {
			dst[key] = value;
		}
	};
	
	// jQuery cross-version bugs fix
	var prop = $.fn.prop ? "prop" : "attr";
	
	//
	// Validators Method
	//
	
	/**
	* Multiple Methods for Validators.
	REMOVE:
	* @param {string} key The validator key
	GET:
	* @param {string} key The validator key
	* @return {object} The found validator, undefined otherwise
	CREATE:
	* @param {string} key The validator key. Optional. In this case, key could be defined
	*  as property in the validator options. 
	* @param {string} key The new validator. Sould be defined as object or function
	* @return {object} The created validator (as object).
	* object.
	* @see createValidatorFromFunction_(key, fct)
	* @see createValidatorFromObject_(key, obj)
	*/
	valid.validators = function(key, validator){
		var isString;
		if(validator === null){
			// Remove
			validator = validators[key];
			delete validators[key];
			return validator;
		} else if((isString = jef.isString(key)) && validator === undefined){
			// Get
			return validators[key];
		} else {
			// Create
			return isString ? 
				createValidator_(key === 'custom' ? null : key, validator) : 
				createValidator_(null, key);
		}
	};
	
	/**
	* Validators container
	*/	
	var validators = {};
	/** Custom validator ids */
	var nextCustomValidatorId_ = 0;	
	/**
	 * Get validator key from class name.
	 * @param {string} The class name
	 * @return {string} The validator key
	 */
	var validatorKeyFromClass_ = function(className){			
		return className.substring(2);
	};
	/**
	* Create validator from function, witch define the 'valid' funcion
	* of validator.
	* @param {string} key The validator key
	* @param {string} key The new validator as function
	* @return {object} The created validator (as object).
	*/
	var createValidatorFromFunction_ = function(key, fct){
		return { 
			key : key || ('custom_' + (++nextCustomValidatorId_)),
			options : {},
			valid : fct
		};
	};
	/**
	* Create validator
	* @param {string} key The validator key
	* @param {string} key The new validator: {
	* 	valid: function(value, options) Valid function, witch realize validation stuff. Required
	* 	parse: function(value, options) Parse function, to parse previously the value to be 
	*		validated. Optional
	* 	options: object Validator options to be used in valid and parse functions. Optional
	* }
	* @return {object} The created validator (as object).
	*/
	var createValidatorFromObject_ = function(key, obj){
		// Check for valid method
		if(!jef.isFunction(obj.valid)){
			return Error("Custom validator object must define a function called 'valid'.");
		}
		// Secure options
		obj.options = jef.isObject(obj.options) ? obj.options : {};
		// Assign key
		obj.key = key || obj.key || ('custom_' + (++nextCustomValidatorId_));
		return obj;
	};
	/** Common function for create validator functions, 
	* dealing with log and error stuff */
	var createValidator_ = function(key, validator){
		if(jef.isFunction(validator)){
			validator = createValidatorFromFunction_(key, validator);
		} else if(jef.isObject(validator)){
			validator = createValidatorFromObject_(key, validator);
		} else {
			log.error('[jef.valid.createValidator] Validator must be a function or an object');
			return;
		}
		// If error
		if(validator instanceof Error){
			log.warn('[jef.valid.createValidator] ' + Error.message);
			return;
		}
		// Check for overwritting validator
		key = validator.key;
		if(validators[key]){
			log.warn('[jef.valid.createValidator] Overwritting existing validator : ' + validator.key);
		}
		// Store validator and return
		return validators[key] = validator;
	};
	
	//
	// Outputs Method
	//
	
	/**
	* Multiple Methods for Validators.
	*/
	valid.outputs = (function(){
		
		var fct = function(field, form){
			var output;
			// Set field output
			if(jef.isString(field)){
				output = outputs.fields[field];
				if(!output){
					jef.log.error('[jef.valid.outputs] No field output found for key : ' + field);
				}
			} else if(jef.isArray(field)) {
				output = createOrExtendOutput_(field[0], field[1], outputs.fields, 'field');
			} else if(jef.isSimpleObject(field)){
				output = createOrExtendOutput_(field, null, outputs.fields, 'field');
			}
			if(output){
				outputs.field = output;
			}
			// Set form output
			if(jef.isString(form)){
				output = outputs.forms[form];
				if(!output){
					jef.log.error('[jef.valid.outputs] No form output found for key : ' + form);
				}
			} else if(jef.isArray(form)) {
				output = createOrExtendOutput_(form[0], form[1], outputs.forms, 'form');
			} else if(jef.isSimpleObject(form)){
				output = createOrExtendOutput_(form, null, outputs.forms, 'form');
			} else { output = null; }
			if(output){
				outputs.form = output;
			}
		};
		
		return jef.extend(fct, {
			field: function(key, output){
				if(key === undefined || key === '') {
					return outputs.field;
				} else if(jef.isString(key) && output === undefined){
					// Get
					return outputs.fields[key];
				} else {
					// Create / Set
					createOrExtendOutput_(key, output, outputs.fields, 'field');
				}
			},
			form: function(key, output){
				if(key === undefined || key === '') {
					return outputs.form;
				} else if(jef.isString(key) && output === undefined){
					// Get
					return outputs.forms[key];
				} else {
					// Create / Set
					createOrExtendOutput_(key, output, outputs.forms, 'form');
				}
			}		
		});
	})();
	
	/**
	* Outputs Container
	*/	
	var outputs = {
		fields : {},
		field : null,
		forms : {},
		form : null,
		keyGen : 0,
		noop : {
			getTarget : jef.noop,
			onValidationStart : jef.noop,
			addErrorMessage : jef.noop,
			addSuccessMessage : jef.noop,
			clearSuccessMessage : jef.noop,
			clearErrorMessage : jef.noop,
			onValidationFail : jef.noop,
			onValidationSuccess : jef.noop
		}
	};	
	
	var createOrExtendOutput_ = function(key, output, _outputs, _key){
		if(!jef.isString(key)){
			output = key;
			key = output.key || (_key + '_' + (++outputs.keyGen));
		}
		var currentOutput = _outputs[key];
		output.key = key;
		if(currentOutput == null){
			// Store
			_outputs[key] = output;
		} else {
			// Merge
			output = jef.extend(true, currentOutput, output);
		}
		return output;
	};

	//
	// Features Method
	//
	
	/**
	* Multiple Methods for Validators. GET, ADD, REMOVE */
	valid.features = function(key, feature){
		if(feature === null){
			// Remove
			feature = features[key];
			delete features[key];
			return feature;
		} else if(feature === undefined){
			// Get
			return features[key];
		} else {
			// Add
			features[key] = feature;
		}
	};
	
	/**
	* Features Container
	*/
	var features = {};
	
	//
	// jQuery functions
	//
	
	$.fn.getFieldValue = function() {
		return this.size() == 0 ? null : getValidationFieldValue_(this);
	};
	$.fn.getFieldValues = function() {
		return this.size() == 0 ? null : getFieldValues_(this);
	};
	$.fn.setFieldValue = function(value) {
		return this.each(function(i, e){
			setFieldValue_($(this), value);
		});
	};
	$.fn.goTo = function(){
		if(this[0]){
			goTo_($(this[0]));
		}
		return this;
	};
	
	/**
	*
	*/
	$.fn.validate = $.fn.valid = function(){
		// Switch on fields/form
		var size = this.size();
		var methods, mainFct, clearFct;
		if(this.filter(':field').size() === size){
			methods = fieldMethods;
			mainFct = fieldMainFct;
			clearFct = fieldClearFct;
		} else if(this.filter('form').size() === size){
			methods = formMethods;
			mainFct = formMainFct;
			clearFct = formClearFct;
		} else {
			log.warn('[$.fn.validate] Validate method must be called on field-only or form-only wrapped set');
			return this;
		}
		// Select Method
		var method = arguments[0], parentArgs = arguments, argsLength = arguments.length, argsIndex = 0;
		if(jef.isString(method)){
			// Stored method
			method = methods[method];
			argsIndex = 1;
		} else if(argsLength == 0){
			// Default method
			method = methods[''];
		} else if(method === null){
			// Clear Function
			method = clearFct;
		} else if(jef.isSimpleObject(method)){
			// Main Function
			method = mainFct;
		} else {
			jef.log.error('[jef.valid.validate] Invalid arguments');
			return this;
		}
		// Check found method
		if(method === undefined){
			jef.log.error('[jef.valid.validate] Invalid method');
			return this;
		}
		// Call method for each element
		var length = min_(argsLength - argsIndex, method[1]), method = method[0], mainResult;
		this.each(function(i, e){
			var args = acopy(parentArgs, argsIndex, [$(e)], 1, length);
			var result = method.apply(this, args);
			if(i === 0 && result !== undefined){
				mainResult = result;
				return false;
			}
		});
		// Return result or wrap set
		return mainResult === undefined ? this : mainResult;
	};
				
	//
	// jQuery selectors
	//
	
	/** :field */
	var rFieldNodeName = /input|select|textarea/i;
	var rFieldNotType = /button|submit|hidden|image/i;
	$.expr[':'].field = function(elem, index, meta, stack){
		return rFieldNodeName.test(elem.nodeName) && !rFieldNotType.test(elem.type);
	};	
	
	var rFocusableNodeName = /input|select|textarea|button|a/i;
	$.expr[':'].focusable = function(elem, index, meta, stack){
		return rFocusableNodeName.test( elem.nodeName ) && !jQuery.expr.filters.hidden( elem );
	};
			
		
	//
	// Validation Data
	//	
	
	/**
	 * Get validation data for a DOM element, and create it if not exist
	 */
	var getValidationData_ = function(we) {
		var validationData = we.data('_validationData');
		if(!validationData){
			// Common, set-once properties
			var id = we[prop]('id'), isForm = we.is('form'), name = we[prop]('name'), nodeName;
			// Check for field name
			if(!isForm && !name){
				jef.log.error('[jef.valid.validationData] Name is undefined for field !');
				return null;
			}
			// Check for id
			if(id){
				id = escapeNW_(id);
			} else{
				// Default id for form (TODO: could not generate id, and manage access to element by name)
				nodeName = we[0].nodeName.toLowerCase();
				id = underscoreNW_(name || ( nodeName + '_jef_' + (++_generatedIdSeq)));
				jef.log.warn(jef.tpl('[jef.valid.validationData] Id isn\'t defined for this {0}. Auto-generated id will be asigned : {1}', 
						[nodeName, id]));
				we[prop]('id', id);
			}
			if(isForm){
				validationData = {
					id: id,
					fieldsSelector: '#' + id + ' :field'
				};
			} else {
				var formId = we.parents('form')[prop]('id');
				if(formId == null){
					jef.log.warn('[jef.valid.validationData] Field should be owned by a form. id = ' + we[prop]('id'));
				} else {
					formId = escapeNW_(formId);
				}
				var isMultiple = isMultiple_(we);
				name = we[prop]('name'), nodeName = we[0].nodeName.toLowerCase();
				var getFieldValueSelector = isMultiple ? (
						(formId ? ('#' + formId + ' ') : 'form ') + nodeName + '[name="' + name + '"]:checked'
					) : ('#' + id);
				var setFieldValueSelector = isMultiple ? (
					(formId ? ('#' + formId + ' ') : 'form ') + nodeName + '[name="' + name + '"]'
				) : ('#' + id);
				validationData = {
					id: id,
					formId: formId,
					isMultiple: isMultiple,
					getFieldValueSelector: getFieldValueSelector,
					setFieldValueSelector: setFieldValueSelector
				};
			}
			we.data('_validationData', validationData);
		}
		return validationData;
	};
	var _generatedIdSeq = 0;
	/**
	* Get a element contained in validation data for a DOM element, and create it if not exist
	*/
	var getValidationElement_ = function(we, key, _v) {
		_v = _v || getValidationData_(we);
		var e = _v[key];
		if(e == null) {
			_v[key] = e = {};
		} 
		return e;
	};
	/**
	* Get a element contained in validation data for a DOM element, and create it as empty array if not exist
	*/
	var getValidationElementArray_ = function(we, key, _v) {
		_v = _v || getValidationData_(we);
		var e = _v[key];
		if(e == null) {
			_v[key] = e = [];
		} 
		return e;
	};
	
	
	/********************************************************************/
	/**************                  FIELDS                **************/
	/********************************************************************/
	
	//
	// Validation Stuff for a DOM element
	//
	
	/***
	 * Build Validation for a DOM element
	 */
	var buildValidation_ = function(we, _v){
		// Fix validation data
		_v = _v || getValidationData_(we);
		if(_v.built == true)
			return _v.builtValidation;				
		// Get validation elements	
		var validationRules = getValidationElement_(we, 'rules', _v);
		var validationFormats = getValidationElement_(we, 'formats', _v);
		var validationOptions = getValidationElement_(we, 'options', _v);
		var validationFeatures = getValidationElement_(we, 'features', _v);
		var validationEvents = getValidationElement_(we, 'events', _v);
		
		// Build Validators and Formatters
		var fieldValidators = [];
		var formatters = [];
		var validator, options, formatter, key;		
		for(key in validationRules){
			validator = validators[key];
			if(validator == null){
				log.warn('[jef.valid.buildValidation] Unfound validator for key : ' + key);
				continue;
			}
			options = validationRules[key] || validator.options || {};
			fieldValidators.push({ 
				key : key,
				validator : validator, 
				options : options,
				condition : parseExpression_(options.condition)
			});
			if(log.isTraceEnabled()){					
				log.trace('[jef.valid.buildValidation] Add validator | id=' + _v.id +
					', validator=' + key);
			}	
		}
		for(key in validationFormats){
			options = validationFormats[key];
			formatter = jef.format.getFormatter(key, options);
			if(formatter == null){
				log.warn('[jef.valid.buildValidation] Unfound formatter for key : ' + key);
				continue;
			}
			// Merge options (to overwrite message)
			options = $.extend({ invalidMessage : formatter.invalidMessage}, options);
			formatters.push({
				key : key,
				formatter : formatter,
				options : options
			});
			if(log.isTraceEnabled()){					
				log.trace('[jef.valid.buildValidation] Add format | id=' + _v.id +
					', format=' + key );
			}	
		}
		
		// Features validation value
		var feature, validationValues = [];
		for(var name in validationFeatures){
			feature = validationFeatures[name];
			if(feature.validationValue){
				validationValues.push(feature.validationValue);
			}
		}	
		var validationValue = validationValues.length == 0 ? null : function(value){
			for(var i=0, l=validationValues.length; i<l; i++){
				value = validationValues[i](value);
			}
			return value;
		};
				
		// Build Validation Object
		_v.builtValidation = {
			// Formatters, validator & events
			formatters : formatters,
			validators : fieldValidators,
			events : validationEvents,
			// Validation features
			validationValue : validationValue,
			// Validation options
			messages : {
				invalidMessage : validationOptions.invalidMessage,
				errorMessage : validationOptions.errorMessage
			},
			condition : parseExpression_(validationOptions.condition),
			// Usefull attribiutes
			empty : fieldValidators.length === 0 && formatters.length === 0 && !validationValue,	
			//isMultiple : _v.isMultiple,
			//formId : _v.formId,
			lastValidValue : null
		};
				
		// Set built flag and return
		_v.built = true;
		return _v.builtValidation;
	};
	/**
	 * Execute validation for a DOM element
	 */
	var executeValidation_ = function(we, output, event, applyFormat, formData){
		
		// Build validation
		var _v = getValidationData_(we);
		var validation = buildValidation_(we, _v);
		if(validation.empty === true)
			return true;
			
		// skipValidationBeforeSubmit
		var formId = _v.formId;
		var formData = formData || getValidationData_( $('#'+formId) );
		var skipValidation = formData.skipValidation === true;
				
		// fix output
		output = output || (formData.formOutputs && formData.formOutputs.field) || outputs.field;
				
		// Local variables
		var eventType = event && event.type;
		var applyFormat = jef.isBoolean(applyFormat) ? applyFormat : (eventType != null ? 
			isIn(eventType, settings.applyFormatEventTypes, true) : true);
		var msgOpt = validation.messages;
		var condition = validation.condition;
		var events = validation.events;		
		// Element and Value
		var domElement = we[0];			
		var value = getValidationFieldValue_(we, _v, validation);
		/*getFieldValue_(we, _v);
		if(validation.validationValue){
			value = validation.validationValue(value);
		}*/
		// Context options
		var parsedValue = null, opt, empty = value.length === 0;
		var eOpt = { 
			field : domElement,
			fieldName : domElement.name, 
			fieldValue : value,
			setFieldValue : function(val){				
				val = setFieldValue_(we, val, _v);
				val = String(val[0] || '');
				if(parsedValue != null && val !== value){
					eOpt.parsedValue = this.parsedValue = parsedValue = null;
				}
				value = this.fieldValue = eOpt.fieldValue = val;				
				empty = value.length === 0;	
			},
			event : event, 
			eventType : eventType,
			formId : formId,
			isEventTarget : event && event.target === domElement,
			lastValidValue : validation.lastValidValue
		};
					
		if(!skipValidation) {
			// Check Condition
			if(validation.condition && validation.condition.call(opt = $.extend({}, eOpt)) === false){
				// Show success message if any			
				if(opt.successMessage){
					output.addSuccessMessage(jef.msg(opt.successMessage, opt), domElement);
				}
				output.clearErrorMessage(domElement);
				return true;
			}		
			// Start event (can modify context options)
			if(eventType && events[eventType + 'Start']){
				if( events[eventType + 'Start'].call(eOpt) === false)
					return false;
			} 
			if(events.start){
				if( events.start.call(eOpt) === false)
					return false;
			}
		}
		
		// Result function
		var result = function(flag, opt){
			var result = flag;
			if(flag){
				// Success events
				if(eventType && events[eventType + 'Success']){
					if( events[eventType + 'Success'].call(opt) === false)
						result = false;
				} 
				if(result && events.success){
					if( events.success.call(opt) === false)
						result = false;
				}
				// Set last valid value if success
				if(result){
					validation.lastValidValue = value;
				}
			} else {
				// Error events
				if(eventType && events[eventType + 'Error']){
					if ( events[eventType + 'Error'].call(opt) === true )
						result = true;
				} 
				if (!result && events.error){
					if( events.error.call(opt) === true)
						result = true;
				}
			}
			if(flag || !result){
				// Complete events
				if(eventType && events[eventType + 'Complete']){
					events[eventType + 'Complete'].call(opt, flag);
				} 
				if(events.complete){
					events.complete.call(opt, flag);
				}
			}
			return result;
		};
				
		// Formatters stuff
		var formatters = validation.formatters, f, formatter, newValue, len = formatters.length;		
		for(var i=0; !empty && i<len; i++){
			f = formatters[i];
			formatter = f.formatter;
			parsedValue = formatter.parse(value);
			if(parsedValue == null){				
				opt = $.extend( {invalidMessage : 'invalid.fieldvalue'}, eOpt, f.options, msgOpt);
				if (!result(false, opt)) {						
					if(!skipValidation){
						output.addErrorMessage(jef.msg(opt.invalidMessage, opt), domElement);
						output.clearSuccessMessage(domElement);
					}
					return false;
				}	
				// Show success message if any			
				if(opt.successMessage && !skipValidation){
					output.addSuccessMessage(jef.msg(opt.successMessage, opt), domElement);
				}
			} else if(applyFormat){
				newValue = formatter.format(parsedValue);
				we.val(newValue);
				eOpt.fieldValue = value = newValue;
				empty = value.length === 0;
			}
			
		}
		
		// Skip here
		if(skipValidation)
			return true;
		
		// Validators stuff
		var validators = validation.validators, v, vld, cond, len = validators.length;
		for(var i=0; i<len; i++){
			v = validators[i];
			vld = v.validator;
			cond = v.condition;
			opt = $.extend( {
				errorMessage : 'incorrect.fieldvalue', 
				invalidMessage : 'invalid.fieldvalue'
				}, eOpt, v.options, msgOpt);
			// Check condition
			if(cond && cond.call(opt) === false){
				continue;
			}			
			// If validator do parse 
			if(vld.parse !== undefined){					
				// Parse if value is not empty
				if(!empty) {
					parsedValue = vld.parse(parsedValue == null ? value : parsedValue, opt);
					if(parsedValue == null && !result(false, opt)){	
						output.addErrorMessage(jef.msg(opt.invalidMessage, opt), domElement);
						output.clearSuccessMessage(domElement);
						return false;
					}
				}
				// Valid if parsedValue is not null
				if(parsedValue != null && vld.valid(parsedValue, opt) !== true && !result(false, opt)){
					output.addErrorMessage(jef.msg(opt.errorMessage, opt), domElement);	
					output.clearSuccessMessage(domElement);					
					return false;
				}
			// Valid from value
			} else if(vld.valid(value, opt) != true && !result(false, opt)){
				output.addErrorMessage(jef.msg(opt.errorMessage, opt), domElement);
				output.clearSuccessMessage(domElement);
				return false;
			}
			// Show success message if any			
			if(opt.successMessage){
				output.addSuccessMessage(jef.msg(opt.successMessage, opt), domElement);
			}
		}		
		
		// Success
		eOpt.parsedValue = parsedValue;
		if(result(true, eOpt)){
			// Show success message if any			
			if(eOpt.successMessage){
				output.addSuccessMessage(jef.msg(eOpt.successMessage, eOpt), domElement);
			}
			output.clearErrorMessage(domElement);
			return true;
		} else {
			// Show error message
			output.addErrorMessage(jef.msg(eOpt.errorMessage || 'incorrect.fieldvalue', eOpt), domElement);
			output.clearSuccessMessage(domElement);
			return false;
		}		
	};
	/**
	* Validate a field
	*/
	var validateField_ = function(we, event, applyFormat){
		// Check element
		if(!checkTag_(we, ['input','select','textarea'], 'jef.valid.validateField')) 
			return true;	
		// Execute Validation
		executeValidation_(we, null, event, applyFormat);
	};
	/**
	* Determine if a field is validate or not
	*/
	var isFieldValid_ = function(we){
		// Check element
		if(!checkTag_(we, ['input','select','textarea'], 'jef.valid.isFieldValid')) 
			return true;
		// Execute Validation
		return executeValidation_(we, outputs.noop, null, false);
	};
	/**
	* Determine if a field is validate or not
	*/
	valid.isFieldValid = function(name, formId){
		return isFieldValid_(wrappFieldName_(name, formId));
	};
	
	//	
	// Validation Process Methods for DOM element
	//
	
	var applyFormat_ = function(we, _v){
		var validation = buildValidation_(we, _v);
		var formatters = validation.formatters, formatter, value, i;
		for(i=0; i<formatters.length; i++){
			formatter = formatters[i].formatter;
			value = getValidationFieldValue_(we, _v, validation);
			if(value === '')
				continue;
			value = formatter.parseAtAllCost(value);
			if(value != null){
				we.val(formatter.format(value));
			}
		}
	};
	
	var addFeatureHandlers_ = function(we, _v, _vfeatures){
		_v.initFeatureEvents = [];
		var feature, featureEvent;
		for(var featureName in _vfeatures){
			feature = _vfeatures[featureName];
			for(var eventType in feature){
				if(!feature.hasOwnProperty(eventType)){
					continue;
				}
				// Add name-spaced handler
				featureEvent = eventType + '.' + featureName;
				we.bind(featureEvent + '_validation', feature[eventType]);				
				// Push to array
				_v.initFeatureEvents.push(featureEvent);
			}
			// Init field
			if(feature.initField){
				feature.initField(we);
			}
		}
	};
	
	var removeFeatureHandlers_ = function(we, _v, _vfeatures){
		var eventTypes = _v.initFeatureEvents, eventType;
		// Remove event handlers
		if(eventTypes != null){
			for(i=eventTypes.length; i--; ){				
				eventType = eventTypes[i];
				// Remove name-spaced event handler
				we.unbind(eventType + '_validation');							
			}
		}		
		// Finalize fields
		var feature;
		for(var featureName in _vfeatures){
			feature = _vfeatures[featureName];
			if(feature.finalizeField){
				feature.finalizeField(we);
			}
		}
		// Delete initFeatureEvents
		delete _v.initFeatureEvents;
	};
	
	var addFieldEventHandlers_ = function(we, _v, _vopt){
		var eventTypes = _vopt.fieldEvents || getDefaultFieldEvents_(we), eventType;
		_v.initFieldsEvents = [];
		for(var i=0, l=eventTypes.length; i<l; i++){			
			eventType = eventTypes[i];
			// Add handler
			we.bind(eventType + '._validation', function(event){
				validateField_($(this), event);
			});
			// Push to array
			_v.initFieldsEvents.push(eventType);				
		}
	};
	
	var removeFieldEventHandlers_ = function(we, _v){
		var eventTypes = _v.initFieldsEvents, eventType;
		if(eventTypes != null){
			for(i=eventTypes.length; i--; ){				
				eventType = eventTypes[i];
				// Remove name-spaced event handler
				we.unbind(eventType + '._validation');							
			}
		}
		// Delete initFieldsEvents
		delete _v.initFieldsEvents;
	};
	
	/**
	* Initialize field validation for a DOM element, adding event handlers.
	*/
	var initFieldValidation_ = function(we, _v){
		// Check element
		if(!checkTag_(we, ['input','select','textarea'], 'jef.valid.initFieldValidation')) 
			return;	
		// Get validation data
		_v = _v || getValidationData_(we);		
		// Skip if init
		if(_v.init == true)
			return;					
		// Initial format
		if(settings.formatOnInitValidation === true){
			applyFormat_(we, _v);
		}	
		// Add Features Handlers
		var _vfeatures = getValidationElement_(we, 'features', _v);
		addFeatureHandlers_(we, _v, _vfeatures);
		// Add Events Handlers
		var _vopt = getValidationElement_(we, 'options', _v);
		addFieldEventHandlers_(we, _v, _vopt);
		// Set init flag
		_v.init = true;
		// Trace
		if(log.isTraceEnabled()){
			log.trace('[jef.valid.initFieldValidation] id = ' + we[prop]('id') + 
				', field events = ' + _v.initFieldsEvents + ', features = ' + _v.initFeatureEvents);
		}
	};
	/**
	* Stop field validation for a DOM element.
	*/
	var stopFieldValidation_ = function(we, _v){
		// Get validation data
		_v = _v || getValidationData_(we);		
		// Remove Features Handlers
		var _vfeatures = getValidationElement_(we, 'features', _v);
		removeFeatureHandlers_(we, _v, _vfeatures);		
		// Remove initalized field events
		removeFieldEventHandlers_(we, _v);				
		// Reset init flag
		_v.init = false;
		// Trace
		if(log.isTraceEnabled()){
			log.trace('[jef.valid.stopFieldValidation] id = ' + we[prop]('id'));	
		}		
	};
	/**
	* Re-initialize field validation for a DOM element, adding event handlers.
	*/
	var reinitFieldValidation_ = function(we, _v){			
		// Get validation data
		_v = _v || getValidationData_(we);
		// Skip if not init
		if(_v.init === false)
			return;		
		// Remove Features Handlers
		var _vfeatures = getValidationElement_(we, 'features', _v);
		removeFeatureHandlers_(we, _v, _vfeatures);		
		// Add Features Handlers		
		addFeatureHandlers_(we, _v, _vfeatures);
		// Remove Field Event Handlers
		removeFieldEventHandlers_(we, _v);
		// Add Field Events Handlers	
		var _vopt = getValidationElement_(we, 'options', _v);
		addFieldEventHandlers_(we, _v, _vopt);		
		// Trace
		if(log.isTraceEnabled()){
			log.trace('[jef.valid.reinitFieldValidation] id = ' + we[prop]('id') + 
				', field events = ' + _v.initFieldsEvents+ ', features = ' + _v.initFeatureEvents);
		}
	};
				
	//
	// Validation Rules for DOM Element
	//
	
	/**
	 * Add validation rule for a DOM element. 
	 */
	var addRule_ = function(we, key, options, _v, _rules){	
		// Check element
		if(!checkTag_(we, ['input','select','textarea'], 'jef.valid.addRule')) 
			return;
		// Get validator
		var validator;
		// Create validator on-the-fly
		if(!jef.isString(key) || key === 'custom'){
			validator = createValidator_(null, key === 'custom' ? options : key);
			if(validator == null){
				log.warn('[jef.valid.addRule] Error creating custom validator');
				return;
			}
			key = validator.key;
			options = null;
		} else {
			validator = validators[key];
			if(validator == null) {
				log.warn('[jef.valid.addRule] No validator was found for key = ' + key );
				return;
			}
		}	
		
		// Get rules options
		_v = _v || getValidationData_(we);
		_rules = _rules || getValidationElement_(we, 'rules', _v);
		var currentOptions = _rules[key];
		if(currentOptions != null){
			log.info('[jef.valid.addRule] Overwritting validation rule | id = ' +
				we[prop]('id') + ', rule =  ' + key + ', current options = '  + 
				jef.format(currentOptions, 'object'));
		}
		// Merge options
		options = $.extend({}, validator.options, currentOptions || {}, options || {});
		// Set options to rule
		_rules[key] = options;
		// Mark as no built
		_v.built = false;		
		// Trace
		if(log.isTraceEnabled()){
			log.trace('[jef.valid.addRule] id = ' +
					we[prop]('id') + ', rule = ' + key);	
		}
	};
	/**
	 * Remove validation rule from DOM element
	 */
	var removeRule_ = function(we, key, _v, _rules){		
		// Get validation data	
		_v = _v || getValidationData_(we);
		if(key == null){
			// Remove all
			_v['rules'] = {};
		} else {
			// Remove rule
			_rules = _rules || _v['rules'];
			if(_rules != null){
				delete _rules[key];
			}		
		}
		// Mark as no built
		_v.built = false;
		// Trace
		if(log.isTraceEnabled()){
			log.trace('[jef.valid.removeRule] id = ' + we[prop]('id') + ', rule = ' + (key ? key : '*'));	
		}
	};
	/** Multiple Method for validation rules: REMOVE, REMOVE ALL, ADD, ADD/REMOVE */
	var rules_ = function(we, key, options, _v){
		_v = _v || getValidationData_(we);
		var _rules = getValidationElement_(we, 'rules', _v);
		var isString = jef.isString(key), opt;
		if(key === null) {
			// Remove all
			removeRule_(we, null, _v, _rules);
		} else if(isString && options === null){
			// Remove
			removeRule_(we, key, _v, _rules);
		} else if(isString){
			// Add
			addRule_(we, key, options, _v, _rules);
		} else {
			// Add / Remove
			options = key;
			for(key in options){
				if((opt = options[key]) === null){
					removeRule_(we, key, _v, _rules);
				} else {
					addRule_(we, key, opt, _v, _rules);
				}
			}			
		}
	};
		
	//
	// Validation Formats for DOM element
	//
		
	/**
	 * Add format for a DOM element
	 */
	var addFormat_ = function(we, key, options, _v, _formats){		
		// Check element
		if(!checkTagsAndType_(we, ['input','textarea'], 'text', 'jef.valid.addFormat'))
			return;
		// Get DOM id
		var domId = we[prop]('id');
		// Fix validation data
		_v = _v || getValidationData_(we);	
		_formats = _formats || getValidationElement_(we, 'formats', _v);
		// Check for overwriting
		if(_formats[key] != null){
			log.info('[jef.valid.addFormat] Overwriting format options | id = ' +
				domElement.id + ', format =  ' + key);
		}		
		// Put format options
		_formats[key] = options;
		// Mark as no built
		_v.built = false;	
		// Apply format if init
		if(_v.init == true && settings.formatOnInitValidation === true){
			applyFormat_(we, _v);
		}		
		// Trace
		if(log.isTraceEnabled()){
			log.trace('[jef.valid.addFormat] id = ' + domId + ', format = '+ formatClass);	
		}
	};
	/**
	 * Remove format from a DOM element
	 */
	var removeFormat_ = function(we, key, _v, _formats){
		// Fix validation data
		_v = _v || getValidationData_(we);
		if(key == null){
			// Remove all
			_v['formats'] = {};
		} else {
			// Remove format	
			_formats = _formats || _v['formats'];
			if(_formats != null){
				delete _formats[key];
			}		
		}	
		// Mark as no built
		_v.built = false;
		// Trace
		if(log.isTraceEnabled()){
			log.trace('[jef.valid.removeFormat] id = ' + we[prop]('id') + ' format = '+ (key ? key : '*'));	
		}
	};
	/** Multiple method for formats. REMOVE, REMOVE ALL, MULTIPLE ADD, ADD/REMOVE */
	var formats_ = function(we, key, options, _v){
		// Validation data
		_v = _v || getValidationData_(we);	
		var _formats = getValidationElement_(we, 'formats', _v);
		var isString = jef.isString(key), isArray, opt;
		if(key === null) {
			// Remove all
			removeFormat_(we, null, _v, _formats);
		} else if(isString && options === null){
			// Remove
			removeFormat_(we, key, _v, _formats);
		} else if(isString || jef.isArray(key)){
			// Multiple Add
			var keys = fixSplitArray_(key, isString);
			for(var i=0; i<keys.length; i++){
				addFormat_(we, keys[i], options, _v, _formats);
			}
		} else {
			// Add / Remove
			options = key;
			for(key in options){
				if((opt = options[key]) === null){
					removeFormat_(we, key, _v, _formats);
				} else {
					addFormat_(we, key, opt, _v, _formats);
				}
			}
		}	
	};
	
	//
	// Validation Options for DOM element
	//
	
	/**
	 * Set message for DOM element
	 */
	var setMessage_ = function(we, key, message, _v, _vopt){
		// Fix validation data
		_v = _v || getValidationData_(we);
		_vopt = _vopt || getValidationElement_(we, 'options',_v);
		// Set message
		_vopt[key] = message;
		// Mark as no built
		_v.built = false;
		// Trace
		if(log.isTraceEnabled()){
			log.trace('[jef.valid.setMessage] id = ' + we[prop]('id') + 
				', key = ' + key + ', message = ' + message);	
		}
	};
	/**
	 * Remove message from DOM element
	 */
	var removeMessage_ = function(we, key, _v, _vopt){
		// Fix validation data
		_v = _v || getValidationData_(we);
		_vopt = _vopt || getValidationElement_(we, 'options', _v);
		// Delete message
		delete _vopt[key];
		// Mark as no built
		_v.built = false;
		// Trace
		if(log.isTraceEnabled()){
			log.trace('[jef.valid.removeMessage] id = ' + we[prop]('id') + ', key = ' + key);	
		}
	};
	/**
	 * Set condition for DOM element
	 */
	var setCondition_ = function(we, condition, _v, _vopt){
		// Fix validation data
		_v = _v || getValidationData_(we);
		_vopt = _vopt || getValidationElement_(we, 'options', _v);		
		// Set condition
		_vopt.condition = condition;
		// Mark as no built
		_v.built = false;
		// Trace
		if(log.isTraceEnabled()){
			log.trace('[jef.valid.setCondition] id = ' + we[prop]('id') + 
				', condition = ' + condition );	
		}
	};
	/**
	 * Remove condition from DOM element
	 */
	var removeCondition_ = function(we, _v, _vopt){
		// Fix validation data
		_v = _v || getValidationData_(we);
		_vopt = _vopt || getValidationElement_(we, 'options', _v);
		// Delete condition
		delete _vopt.condition;
		// Mark as no built
		_v.built = false;
		// Trace
		if(log.isTraceEnabled()){
			log.trace('[jef.valid.removeCondition] id = ' + we[prop]('id'));	
		}
	};
	/**
	 * Set field events for DOM element
	 */
	var setFieldEvents_ = function(we, eventNames, _v, _vopt, _vinit){
		// Fix validation data
		_v = _v || getValidationData_(we);
		_vopt = _vopt || getValidationElement_(we, 'options', _v);
		_vinit = _vinit === undefined ? _v.init : _vinit;
		// Fix eventNames
		eventNames = fixSplitArray_(eventNames);
		// Set events
		_vopt.fieldEvents = eventNames;		
		// Trace
		if(log.isTraceEnabled()){
			log.trace('[jef.valid.setFieldEvents] id = ' + we[prop]('id') + 
				', events = ' + eventNames);	
		}
		// Reinit events handlers
		if(_vinit == true){
			reinitFieldValidation_(we, _v);
		}
	};
	/**
	 * Remove field events from DOM element
	 */
	var removeFieldEvents_ = function(we, _v, _vopt, _vinit){
		// Fix validation data
		_v = _v || getValidationData_(we);
		_vopt = _vopt || getValidationElement_(we, 'options', _v);
		_vinit = _vinit === undefined ? _v.init : _vinit;
		// Delete events (set as empty array because undefefined value
		// will be replaced by defaultFieldsEvents when initializing events 
		_vopt.fieldEvents = [];
		// Trace
		if(log.isTraceEnabled()){
			log.trace('[jef.valid.removeFieldEvents] id = ' + we[prop]('id'));	
		}
		// Reinit events handlers
		if(_vinit == true){
			reinitFieldValidation_(we, _v);
		}
	};
				
	/**
	 * Set validation options for a DOM element
	 */
	var setValidationOptions_ = function(we, options, _v,  _vopt){				
		// Check element
		if(!checkTag_(we, ['input','select','textarea'], 'jef.valid.setValidationOptions')) 
			return;	
		// Get validation options
		_v = _v || getValidationData_(we);
		_vopt = _vopt || getValidationElement_(we, 'options', _v);
		var reinit = false;
		
		// Invalid message
		if(options.invalidMessage){
			setMessage_(we, 'invalidMessage', options.invalidMessage, _v, _vopt);
		} else if(options.invalidMessage === null) {
			removeMessage_(we, 'invalidMessage', _v, _vopt);
		}	
		// Error message
		if(options.errorMessage){
			setMessage_(we, 'errorMessage', options.errorMessage, _v, _vopt);
		} else if(options.errorMessage === null){
			removeMessage_(we, 'errorMessage', _v, _vopt);
		}		
		// Condition
		if(options.condition){
			setCondition_(we, options.condition, _v, _vopt);
		} else if(options.condition === null){
			removeCondition_(we, _v, _vopt);
		}			
		// Field Events
		if(options.fieldEvents){
			setFieldEvents_(we, options.fieldEvents, _v, _vopt, false); 
			reinit = true;
		} else if(options.fieldEvents === null){
			removeFieldEvents_(we, _v, _vopt, false); 
			reinit = true;
		}				
		// Reinit field validation if init
		if( reinit && _v.init === true ){
			reinitFieldValidation_(we, _v);
		}
	};
	/**
	* Remove validation options from a DOM element
	*/
	var removeValidationOptions_ = function(we, _v,  _vopt){
		// Get validation options
		_v = _v || getValidationData_(we);
		_vopt = _vopt || getValidationElement_(we, 'options', _v);		
		// Remove Options
		removeMessage_(we, 'invalidMessage', _v, _vopt);
		removeMessage_(we, 'errorMessage', _v, _vopt);
		removeCondition_(we, _v, _vopt);
		removeFieldEvents_(we, _v, _vopt, false);
		// Reinit field validation if init
		if( _v.init == true ){
			reinitFieldValidation_(we, _v);
		}
	};	
	/** Multiple method for formats. REMOVE, REMOVE ALL, ADD, ADD/REMOVE */
	var options_ = function(we, key, options, _v){
		// Validation data
		_v = _v || getValidationData_(we);	
		var _vopt = getValidationElement_(we, 'options', _v);
		var isString = jef.isString(key), isArray, opt;
		if(key === null) {
			// Remove all
			removeValidationOptions_(we, _v, _vopt);
		} else if(isString){
			// Add or Remove
			options = {}; options[key] = options;
			setValidationOptions_(we, options, _v, _vopt);
		} else {
			// Add / Remove
			setValidationOptions_(we, options, _v, _vopt);
		}	
	};
				
	//
	// Validation Features for DOM element
	//
	
	/**
	 * Set validation features for a DOM element
	 */
	var setValidationFeatures_ = function(we, features, _v, _vfeatures){				
		// Check element
		if(!checkTag_(we, ['input','select','textarea'], 'jef.valid.setValidationFeatures')) 
			return;	
		// Get validation options
		_v = _v || getValidationData_(we);
		_vfeatures = _vfeatures || getValidationElement_(we, 'features', _v);
		var reinit = false;		
		//  Add/Remove features
		for(var name in features){
			var options = features[name];
			if( options === null ) {
				reinit |= removeFeature_(we, name, _v, _vfeatures, false);
			} else {
				reinit |= addFeature_(we, name, options, _v, _vfeatures, false);
			}	
		}		
		// Reinit field validation if init
		if( reinit && _v.init == true ){
			reinitFieldValidation_(we, _v);
		}
	};	
	/**
	 * Remove all validation features for a DOM element
	 */
	var removeFeatures_ = function(we, _v, _vfeatures){
		// Get validation options
		_v = _v || getValidationData_(we);
		_vfeatures = _vfeatures || getValidationElement_(we, 'features', _v);
		var reinit = false;	
		//  Remove features
		for(var name in _vfeatures){
			reinit |= removeFeature_(we, name, _v, _vfeatures, false);
		}		
		// Reinit field validation if init
		if( reinit && _v.init == true ){
			reinitFieldValidation_(we, _v);
		}		
	};
	/**
	* Add validation feature for a DOM element
	*/
	var addFeature_ = function(we, name, options, _v, _vfeatures, _vinit){
		// Fix validation data
		_v = _v || getValidationData_(we);
		_vfeatures = _vfeatures || getValidationElement_(we, 'features', _v);
		_vinit = _vinit === undefined ? _v.init : _vinit;		
		// Get feature definition
		var featureDef = features[name];
		if(featureDef == null){
			log.warn('[jef.valid.addFeature] No feature is defined : ' + name);
			return false;
		}		
		// Get current feature
		var currentFeature = _vfeatures[name];
		if(currentFeature){
			// Finalize it
			if(currentFeature.finalizeField){
				currentFeature.finalizeField(we);
			}
			log.info('[jef.valid.addFeature] Overwritting feature : name = "' + name + ', id = ' + we[prop]('id'));
		}
		// Parse options	
		if(isFct(featureDef.parseOptions)){
			options = featureDef.parseOptions(options);
		}
		// Init field
		var initField;
		if(isFct(featureDef.initField)){
			initField = function(field){
				return featureDef.initField(options, field);
			};
		}
		// Finalize field
		var finalizeField;
		if(isFct(featureDef.finalizeField)){
			finalizeField = function(field){
				return featureDef.finalizeField(options, field);
			};
		}
		// Validation value
		var validationValue;
		if(isFct(featureDef.validationValue)){
			validationValue = function(value){
				return featureDef.validationValue(options, value);
			};
		}
		// Build feature
		var feature = jef.delegate({
			options : options,
			initField : initField,
			finalizeField : finalizeField,
			validationValue : validationValue			
		});
		var rskipKeys = /parseOptions|validationValue|initField|finalizeField/;
		for(var key in featureDef){
			if(rskipKeys.test(key)){
				continue;
			}
			feature[key] = function(featureFct){
				return function(event){
					return featureFct.call(this, options, event);
				};
			}(featureDef[key]);
		}
		// Set feature
		_vfeatures[name] = feature;	
		// Mark as no built
		_v.built = false;
		// Trace
		if(log.isTraceEnabled()){
			log.trace('[jef.valid.addFeature] id = ' + we[prop]('id') + 
				', name = ' + name + ', options = ' + stringOptions);	
		}
		// Reinit events handlers
		if(_vinit == true){
			reinitFieldValidation_(we, _v);
		}
		return true;
	};
	/**
	* Remove validation feature for a DOM element
	*/
	var removeFeature_ = function(we, name, _v, _vfeatures, _vinit){
		// Fix validation data
		_v = _v || getValidationData_(we);
		_vfeatures = _vfeatures || getValidationElement_(we, 'features', _v);
		_vinit = _vinit === undefined ? _v.init : _vinit;		
		// Finalize feature
		var feature = _vfeatures[name];
		if(feature && feature.finalizeField){
			feature.finalizeField(we);
		}	
		// Delete feature
		var result = delete _vfeatures[name];	
		// Mark as no built
		_v.built = false;
		// Trace
		if(result && log.isTraceEnabled()){
			log.trace('[jef.valid.removeFeature] id = ' + we[prop]('id') + 
				', name = ' + name);	
		}
		// Reinit events handlers
		if(_vinit == true){
			reinitFieldValidation_(we, _v);
		}
		return result;
	};
	/** Multiple Method for validation features: REMOVE, REMOVE ALL, ADD, ADD/REMOVE */
	var features_ = function(we, key, options, _v){
		_v = _v || getValidationData_(we);
		var _vfeatures = getValidationElement_(we, 'features', _v);
		var isString = jef.isString(key), opt;
		if(key === null) {
			// Remove all
			removeFeatures_(we, _v, _vfeatures);
		} else if(isString && options === null){
			// Remove
			removeFeature_(we, key, _v, _vfeatures);
		} else if(isString){
			// Add
			addFeature_(we, key, options, _v, _vfeatures);
		} else {
			// Add / Remove
			options = key;
			for(key in options){
				if((opt = options[key]) === null){
					removeFeature_(we, key, _v, _vfeatures);
				} else {
					addFeature_(we, key, opt, _v, _vfeatures);
				}
			}			
		}
	};
			
	//
	// Validation Events for DOM element
	//
	
	/**
	 * Add validation event for a DOM element
	 */
	var addEvent_ = function(we, name, event, _v, _events){				
		// Check element
		if(!checkTag_(we, ['input','select','textarea'], 'jef.valid.addValidationEvent')) 
			return;	
		// Set validation events
		_v = _v || getValidationData_(we);
		_events = _events || getValidationElement_(we, 'events', _v);
		// Check for overwrite
		if(_events[name] != null){
			log.info('[jef.valid.addValidationEvent] Overwritting validation event | id = ' +
				we[prop]('id') + ', event = ' + name);
		}
		// Add Event
		_events[name] = event;
		// Mark as no built
		_v.built = false;
		// Trace
		if(log.isTraceEnabled()){
			log.trace('[jef.valid.addValidationEvent] id = ' + we[prop]('id') + 
				', event = ' + name);	
		}
	};
	/**
	* Remove validation event from DOM element
	*/
	var removeEvent_ = function(we, name, _v, _events){		
		// Remove validation event
		_v = _v || getValidationData_(we);
		_events = _events || getValidationElement_(we, 'events', _v);
		if(name == null){
			_v['events'] = {};
		} else {
			delete _events[name];
		}
		// Mark as no built
		_v.built = false;
		// Trace
		if(log.isTraceEnabled()){
			log.trace('[jef.valid.removeValidationEvent] id = ' + we[prop]('id') + 
				', event = ' + name);	
		}
	};
	/** Multiple Method for validation rules: REMOVE, REMOVE ALL, ADD, ADD/REMOVE */
	var events_ = function(we, key, options, _v){
		_v = _v || getValidationData_(we);
		var _events = getValidationElement_(we, 'events', _v);
		var isString = jef.isString(key), opt;
		if(key === null) {
			// Remove all
			removeEvent_(we, null, _v, _events);
		} else if(isString && options === null){
			// Remove
			removeEvent_(we, key, _v, _events);
		} else if(isString){
			// Add
			addEvent_(we, key, options, _v, _events);
		} else {
			// Add / Remove
			options = key;
			for(key in options){
				if((opt = options[key]) === null){
					removeEvent_(we, key, _v, _events);
				} else {
					addEvent_(we, key, opt, _v, _events);
				}
			}			
		}
	};
		
	//
	// Validation Debug for DOM element
	//	
		
	/** 
	* Get displayable, built validation for a field. 
	* This function is made especialy for debugging.
	*/
	var getFieldValidation_ = function(we){
		// Fix validation data
		var _v = getValidationData_(we);		
		var bv = buildValidation_(we, _v);
		// Build validation object
		return {
			rules : function(){
				var v = {}, vtr = bv.validators;
				for(var i=0, len=vtr.length; i<len; i++){
					v[vtr[i].key] = vtr[i].options;
				}
				return v;
			}(),
			formats : function(){
				var f = {}, ftr = bv.formatters;
				for(var i=0, len=ftr.length; i<len; i++){
					f[ftr[i].key] = ftr[i].options;
				}
				return f;
			}(),
			options : function(){
				return {
					invalidMessage : bv.messages.invalidMessage,
					errorMessage : bv.messages.errorMessage,
					condition : _v['options'].condition,
					fieldEvents : _v['options'].fieldEvents
				};
			}(),
			features : function(){
				var features = _v['features'], featureName, featuresOptions = {};
				for(featureName in features){
					featuresOptions[featureName] = features[featureName].options;
				}
				return featuresOptions;
			}(),
			events : bv.events
		};	
	};
	
	var showFieldValidation_ = function(we, output){
		var validation = getFieldValidation_(we);
		output = output || jef.log.info;
		output(jef.format(validation, 'object', { 
			indent: true, deep: true, indentArray: false, skipNull: true, 
			name:  "Validation for field [" + we[prop]('id') + "]"
		}));
	};
		
	//	
	// $.fn.validate field implementation
	//
	
	var fieldMethods = {
		// default (no-args)
		'': [validateField_, 2],
		// Validation process
		init: [initFieldValidation_, 0],
		stop: [stopFieldValidation_, 0],
		reinit: [reinitFieldValidation_, 0],
		validate: [validateField_, 2],
		test: [isFieldValid_, 0],
		// Debug
		show: [showFieldValidation_, 1],
		// Sub-elements
		rules: [rules_, 2],
		formats: [formats_, 2],
		options: [options_, 2],
		features: [features_, 2],
		events: [events_, 2]
	};
	/**
	* Main method for field validation have the next structure :
	* {
	*	rules: {
	* 		validatorKey: <object> (validator options) {
	*			invalidMessage: <string>,
	*			errorMessage: <string>,
	*			condition: <string> || <function> (@see expression parser),
	*			... (validator specific attributes)
	* 		}
	*	},
	*	formats: <string> (space-separated format keys) || <object> {
	* 		formatKey: <object> (feature options)
	* 	},
	*	options: {
	*		invalidMessage: <string>,
	*		errorMessage: <string>,
	*		condition: <string> || <function> (@see expression parser),
	*		fieldEvents: <string> (space separated jquery event names)
	*	},
	*	features: {
	*		featureKey: <object> (feature options)
	*	},
	*	events: {
	*		validationEventName: <function> (event handler)
	*	},
	*	... (other attributes will be interpreted as rules)
	* }
	*/
	var fieldMainFct = [function(we, validations){
		var options = null;
		var _v = getValidationData_(we);
		for(key in validations){
			options = validations[key];
			if(key == 'options'){
				setValidationOptions_(we, options, _v);
			} else if (key == 'features'){
				setValidationFeatures_(we, options, _v);
			} else if (key === 'events'){
				events_(we, options, undefined, _v);
			} else if (key === 'formats'){
				formats_(we, options, undefined, _v);
			} else if( key === 'rules'){
				rules_(we, options, undefined, _v);
			} else {
				addRule_(we, key, options, _v);
			}
		}
	}, 1];
	var fieldClearFct = [function(we){
		// Fix validation data
		var _v = getValidationData_(we);
		// Delete all validation elements
		rules_(we, null, undefined, _v);
		formats_(we, null, undefined, _v);
		options_(we, null, undefined, _v);
		features_(we, null, undefined, _v);
		events_(we, null, undefined, _v);
		// Mark as no built
		_v.built = false;
		// Trace
		if(log.isTraceEnabled()){
			log.trace('[jef.valid.removeValidation] id = ' + we[prop]('id'));	
		}
	}, 0];
	
	/********************************************************************/
	/**************                  FORM                  **************/
	/********************************************************************/
	
	//
	// Validation Stuff
	//

	/**
	* Validate a form
	*/
	var validateForm_ = function(we, event, options, output, _v){		
		// Get validation data
		_v = _v || getValidationData_(we);
		// skipValidationBeforeSubmit
		_v.skipValidation = false;
		// Fix options 					
		options = jef.extend({}, getValidationElement_(we, 'formOptions', _v), options || {});
		// Fix output
		output = output || (_v.formOutputs && _v.formOutputs.form) || outputs.form;
		// Fix events
		var events = _v.formEvents || {};
		// Build context options
		var formId = _v.id, errorCount = 0, errorData = [];
		options = $.extend({
				stopOnFirstError : settings.stopFormValidationOnFirstError,
				fixFieldValidation : settings.fixFieldsValidationWithForm
			}, options, {
				form: we[0],
				formId: formId,
				output: output,
				target: null,
				event: event,
				eventType: event && event.type
		});
		
		// Create error appender
		var ea = {			
			addErrorMessage : function(message, domElement){
				errorCount++;
				errorData.push({ 
					message : message,
					fieldName : domElement && domElement.name
				});
				output.addErrorMessage(options.target, message, domElement);
			},
			addSuccessMessage : function(message, domElement){
				output.addSuccessMessage(options.target, message, domElement);
			},
			clearErrorMessage : function(domElement){				
				output.clearErrorMessage(options.target, domElement);
				var fieldName = domElement && domElement.name;
				if( fieldName ){
					for(var i = errorData.length; i--; ){
						if(errorData[i].fieldName === fieldName){
							errorCount--;				
							errorData.splice(i,1);
							break;
						}	
					}
				}				
			},
			clearSuccessMessage : function(domElement){
				output.clearSuccessMessage(options.target, domElement);
			}
		};	
		
		//
		// Build form Validators
		//		
		var formValidators = [];
		var successValueModifiers = [];
		// Push field validators		
		var multipleInputs = {}, fixFieldValidation = options.fixFieldValidation ;		
		$( _v.fieldsSelector ).each(function(i,e){			
			// Wrap input element
			var wi = $(e), _vfield = getValidationData_(wi);	
			// Fix field validation
			if( fixFieldValidation ) {
				reinitFieldValidation_(wi, _vfield);
			}
			// Build validation
			var validation = buildValidation_(wi, _vfield);			
			if(validation.empty == true)
				return;	
			// Multiple inputs
			if( _vfield.isMultiple ){				
				if(multipleInputs[e.name] == true)
					return;
				multipleInputs[e.name] = true;
			}	
			// Create success value modifier
			if(validation.validationValue){
				successValueModifiers.push(function(){
					var value = getValidationFieldValue_(wi, _vfield, validation);
					setFieldValue_(wi, value, _vfield);
				});	
			}
			// Create form validator
			formValidators.push( function(ea){
				return executeValidation_(wi, ea, this.event, null, _v);			
			});
		});		
		// Push form validators
		aconcat(formValidators, getValidationElementArray_(we, 'formValidations', _v));
			
		//
		// Validate
		//		
			
		// Get target
		options.target = output.getTarget(formId) || formId;			
		// Start Events
		output.onValidationStart(options.target);
		if(events.start && events.start.call(options, ea) === false){
			return false;
		}
		// Execute validations
		var success = true, len = formValidators.length;
		if(options.stopOnFirstError){
			for(var i=0; i<len && success; i++){
				if(!formValidators[i].call(options, ea)){
					success = false;
				} 
			}
		} else {
			for(var i=0; i<len; i++){
				if(!formValidators[i].call(options, ea)){
					success = false;
				} 
			}
		}	
		// Set validation report
		options.errorCount = errorCount;
		options.errorData = errorData;		
		// Success/Error Events
		if( !success ){
			if(events.error && events.error.call(options, ea) === true){
				output.onValidationSuccess(options.target, options);
				success = true;
			} else {
				output.onValidationFail(options.target, options);
			}			
		} else {
			if(events.success && events.success.call(options, ea) === false){
				output.onValidationFail(options.target, options);
				success = false;
			} else {
				output.onValidationSuccess(options.target, options);
			}
		}			
		// Complete event
		if(events.complete) {
			events.complete.call(options, ea, success);
		}
		// Clean possible stored domElements
		delete options.form;
		delete options.target;
		// Feature Validation Value
		if( success ){
			for(var i=0, l=successValueModifiers.length; i<l; i++){
				successValueModifiers[i]();
			}
		}		
		// Return 'success' flag
		return success;			
		
	};
	
	var isFormValid_ = function(we, options){
		// Execute Validation 
		options = options || {};
		options.stopOnFirstError = true;
		return validateForm_(this, null, options, outputs.noop);
	};
	
	//	
	// Validation Process Methods for DOM element
	//
		
	/**
	* Initialize form validation, adding onSubmit handler.
	* @param {string|domElement} domElementSelector jQuery selector for getting the form.
	* @param {boolean} initFieldsValidation If form fields (input, ...) will be initialize too.
	*/
	var initFormValidation_ = function(we, initFieldsValidation, _v) {
		// Get validation data
		_v = _v || getValidationData_(we);
		var _options = getValidationElement_(we, 'formOptions', _v);
		// skipValidationBeforeSubmit
		if(_v.skipValidation === undefined){
			_v.skipValidation = _options.skipValidationBeforeSubmit !== undefined ? 
				_options.skipValidationBeforeSubmit : settings.skipValidationBeforeSubmit;
		}
		// Fix arguments
		if(!jef.isBoolean(initFieldsValidation)){
			initFieldsValidation = _options.initFieldsValidationWithForm !== undefined ? 
				_options.initFieldsValidationWithForm : settings.initFieldsValidationWithForm;
		}		
		// Add on submit event
		if( _v.formInit != true ){
			we.bind('submit._validation', function(event){
				return validateForm_($(this), event);
			});	
			// Trace
			if(log.isTraceEnabled()){
				log.trace('[jef.valid.initFormValidation] id = ' + we[prop]('id'));	
			}		
		}
		// Init fields validation
		if(initFieldsValidation){
			$( _v.fieldsSelector ).each(function(i,e){
				initFieldValidation_($(e));
			});
		}	
		// Set init flag
		_v.formInit = true;
	};
	/**
	* Stop form validation, removing onSubmit handlers
	*/
	var stopFormValidation_ = function(we, stopFieldsValidation, _v) {
		_v = _v || getValidationData_(we);
		// Unbind submit event
		we.unbind('submit._validation');	
		// Trace
		if(log.isTraceEnabled()){
			log.trace('[jef.valid.stopFormValidation] id = ' + we[prop]('id'));	
		}		
		// Stop fields validation
		stopFieldsValidation = jef.isDef(stopFieldsValidation) ? stopFieldsValidation : 
			settings.stopFieldsValidationWithForm;
		if(stopFieldsValidation){
			$( _v.fieldsSelector ).each(function(i,e){
				stopFieldValidation_($(e));
			});
		}
		// Reset init flag
		_v.formInit = false;
	};
	
	//
	// Form Validations
	//
	
	/** Add validations for a field, given his name, in a form */	
	var addFormFieldValidations_ = function(we, fieldName, fieldValidations, _v) {
		_v = _v || getValidationData_(we);
		wrappFieldName_(fieldName, _v.id).validate(fieldValidations);
	};
		
	/** Add validations for form fields */
	var formFieldsValidations_ = function(we, fieldsValidations, _v){
		_v = _v || getValidationData_(we);
		for(var fieldName in fieldsValidations) {
			addFormFieldValidations_(we, fieldName, fieldsValidations[fieldName], _v);
		}
	};
		
	/** Multiple method for form validations. REMOVE ALL, MULTIPLE ADD, ADD */	
	var formValidations_ = function(we, formValidations, _v){
		// Validation data
		var _formValidations = getValidationElementArray_(we, 'formValidations', _v);
		if(formValidations === null) {
			// Remove all
			_formValidations.splice(0, _formValidations.length);
		} else if(jef.isFunction(formValidations)){
			// Add
			_formValidations.push(formValidations);
		} else if(jef.isArray(formValidations)){
			// Multiple Add
			aconcat(_formValidations, formValidations);
		} 
	};
	
	//
	// Form Validation Options
	//
	
	/** Multiple method for form options. GET, REMOVE, REMOVE ALL, ADD, MULTPIPLE ADD/REMOVE */
	var formOptions_ = function(we, key, options, _v){
		// Validation data
		_v = _v || getValidationData_(we);	
		var _options = getValidationElement_(we, 'formOptions', _v);
		var isString = jef.isString(key), opt;
		if(key === null) {
			// Remove all
			_v.formOptions = {};
		} else if(isString && options === null){
			// Remove
			delete _options[key];
		} else if(isString && options === undefined) {
			// Get
			return _options[key];
		} else if(isString){
			// Add
			_options[key] = options;
		} else if(jef.isSimpleObject(key)){
			// Multiple Add/Remove
			options = key;
			for(key in options){
				if((opt = options[key]) === null){
					delete _options[key];
				} else {
					_options[key] = opt;
				}
			}
		}	
	};
	
	//
	// Form Validation Events
	//
	
	/** Multiple method for form options. GET, REMOVE, REMOVE ALL, ADD, MULTPIPLE ADD/REMOVE */
	var formEvents_ = function(we, key, options, _v){
		// Validation data
		_v = _v || getValidationData_(we);	
		var _events = getValidationElement_(we, 'formEvents', _v);
		var isString = jef.isString(key), opt;
		if(key === null) {
			// Remove all
			_v.formEvents = {};
		} else if(isString && options === null){
			// Remove
			delete _events[key];
		} else if(isString && options === undefined) {
			// Get
			return _events[key];
		} else if(isString) {
			// Add
			_events[key] = options;
		} else if(jef.isSimpleObject(key)){
			// Multiple Add/Remove
			options = key;
			for(key in options){
				if((opt = options[key]) === null){
					delete _events[key];
				} else {
					_events[key] = opt;
				}
			}
		}	
	};
		
	//
	// Form Validation Outputs
	//
	
	var addFormOutput_ = function(we, type, output, _outputs, _v){
		if(type !== 'field' && type !== 'form'){
			jef.log.error('[jef.valid.addFormOutput] FormOutputs type must be one of those : \'form\', \'field\'');
			return;
		}
		_outputs = _outputs || getValidationElement_(we, 'formOutputs', _v);
		var typeOutputs = outputs[type + 's'], key, options;
		if(jef.isString(output)){
			key = output;			
		} else if(jef.isArray(output)){
			key = output[0];
			options = output[1];
		} else {
			jef.log.error('[jef.valid.addFormOutput] Invalid Arguments');
			return;
		}
		output = typeOutputs[key];
		if(!output){
			jef.log.error('[jef.valid.addFormOutput] No ' + type + ' output found for key : ' + key);
			return;
		}
		// Set options
		if(options != null){
			// Delegate output in order to protect source output options
			output = jef.delegate(output);
			// Set new extended options
			output.options = jef.extend({}, output.options, options);
		}
		// Set to form data
		_outputs[type] = output;
	};
	
	/** Multiple method for form outputs. GET, REMOVE, REMOVE ALL, ADD, MULTPIPLE ADD/REMOVE */
	var formOutputs_ = function(we, key, output, _v){
		// Validation data
		_v = _v || getValidationData_(we);	
		var _outputs = getValidationElement_(we, 'formOutputs', _v);
		var isString = jef.isString(key), opt;
		if(key === null) {
			// Remove all
			_v.formOutputs = {};
		} else if(isString && output === null){
			// Remove
			delete _outputs[key];
		} else if(isString && output === undefined) {
			// Get, default to page outputs
			return _outputs[key] || outputs[key];
		} else if(isString){
			// Add
			addFormOutput_(we, key, output, _outputs, _v);
		} else if(jef.isSimpleObject(key)){
			// Multiple Add/Remove
			options = key;
			for(key in options){
				if((opt = options[key]) === null){
					delete _outputs[key];
				} else {
					addFormOutput_(we, key, opt, _outputs, _v);
				}
			}
		}	
	};
		
	//
	// Form Validation Debug
	//	
	
	var showFormValidation_ = function(we, output, _v){
		// Each showFormValidation for field
		_v = _v || getValidationData_(we);
		var names = {}, rules = {};
		$( _v.fieldsSelector ).each(function(i, e){
			// Check for unique name
			if(rules[e.name])
				return;
			// Build field validation rules
			rules[e.name] = getFieldValidation_($(e));
		});			
		// Output rules
		output = output || jef.log.info;
		output(jef.format(rules, 'object', { 
			indent: true, deep: true, indentArray: false, skipNull: true,
			name:  "Validation for form [" + _v.id + "]"
		}));
	};
	
	//
	// $.fn.validate form implementation
	//
	
	var formMethods = {
		// default (no-args)
		'': [validateForm_, 3],			
		// Validation process
		init: [initFormValidation_, 1],
		stop: [stopFormValidation_, 1],
		validate: [validateForm_, 3],
		test: [isFormValid_, 1],
		// Debug
		show: [showFormValidation_, 1],
		// Sub-elements
		fields: [formFieldsValidations_, 1],
		formValidations: [formValidations_, 1],
		options: [formOptions_, 2],
		outputs: [formOutputs_, 2],
		events: [formEvents_, 2]
	};	
	/**
	* Main method for form validation have the next structure :
	* {
	*	fields: {
	* 		fieldName: <object> (see main method for field validation)
	*	},
	*	formValidations: <function> || <array[function ...]> (form-based validation),
	*	options: {
	*		stopOnFirstError: true|false,
	*		fixFieldValidation: true|false,
	*	},
	*	outputs: {
	*		field: <string> (field output key) || <array[<string>, <object>]> field output key and options,
	*		form: <string> (form output key) || <array[<string>, <object>]> form output key and options
	*	}
	*	events: {
	*		validationEventName: <function> (event handler)
	*	},
	*	... (other attributes will be interpreted as fields)
	* }
	*/
	var formMainFct = [function(we, validations){
		var options = null;
		var _v = getValidationData_(we);
		for(key in validations){
			options = validations[key];
			if(key == 'fields'){
				formFieldsValidations_(we, options, _v);
			} else if (key == 'formValidations'){
				formValidations_(we, options, _v);
			} else if (key === 'options'){
				formOptions_(we, options, undefined, _v);
			} else if (key === 'outputs'){
				formOutputs_(we, options, undefined, _v);
			} else if( key === 'events'){
				formEvents_(we, options, undefined, _v);
			} else {
				addFormFieldValidations_(we, key, options, _v);
			}
		}
	}, 1];
	var formClearFct = [function(we, _v){
		// Fix validation data
		_v = _v || getValidationData_(we);
		// Delete all validation elements
		$( _v.fieldsSelector ).validate(null);
		formValidations_(we, null, _v);
		formOptions_(we, null, undefined, _v);
		formOutputs_(we, null, undefined, _v);
		formEvents_(we, null, undefined, _v);
		// Trace
		if(log.isTraceEnabled()){
			log.trace('[jef.valid.removeFormValidation] id = ' + _v.id);	
		}
	}, 0];
	
	/********************************************************************/
	/**************      DOM-Ready Initialization          **************/
	/********************************************************************/
		
	$(function(){
				
		// Parse format and validator css class
		if(settings.parseFieldsOnDomReady === true) {
			$(':field[class^="v_"], :field[class^="f_"]').each(function(i, e){
				var we = $(e);
				var classNames = we.attr('class').split(' ');
				var formats = [], rules = {}, hasRule = false;
				for(var i=0; i<classNames.length; i++){
					var className = classNames[i];
					if(className.indexOf('f_') === 0){
						formats.push(formatKeyFromClass_(className));
					} else if(className.indexOf('v_') === 0 && className !== 'v_init'){
						hasRule = true;
						rules[validatorKeyFromClass_(className)] = {};
					}
				}
				if(formats.length > 0){
					we.validate('formats', formats);
				}
				if(hasRule){
					we.validate('rules', rules);
				}
			});
		}
		
		// Init all forms with class 'v_init'
		if(settings.parseFormsOnDomReady === true){
			$('form.v_init').validate('init');
		}
		
		// Init all fields with class 'v_init'
		if(settings.parseFieldsOnDomReady === true) {
			$(':field.v_init').validate('init');
		}
		
		// Overwrite jquery datepicker functions
		if(settings.overwriteDatepickerFormat === true && $.datepicker != null) {
			var jef_format = jef.date.format, jef_parse = jef.date.parse;
			$.datepicker.formatDate = function(format, date, settings){
				return jef_format(date, format);
			};
	
			$.datepicker.parseDate = function(format, value, settings){
				return jef_parse(value, format);
			};			
		}
		
	});
	
	/********************************************************************/
	/**************         Built-in Validators            **************/
	/********************************************************************/
	
	validators['required'] = {
			options : {
				trim : true,
				errorMessage : 'El campo es requerido.' 
			},
			valid : function(value, options){
				if(options.trim == true)
					value = trim(value);
				return value.length != 0;
			}
	};
	
	validators['requiredstring'] = validators['required'];
			
	validators['equalsTo'] = {
			options : {
				compareField : null,
				formId : undefined,
				errorMessage : 'Ese campo debe tener el mismo valor que el campo "{compareField}".' 
			},
			valid : function(value, options){
				if( options.compareField == null )
					return true;
				return valid.getFieldValue(options.compareField, options.formId) === value;				
			}
	};		
			
	validators['stringlength'] = {
			options : {
				minLength : null,
				maxLength : null,
				trim : true,
				errorMessage : 'No debe ingresar más de {maxLength} caracteres' 
			},
			valid : function(value, options){	
				if(options.trim == true)
					value = trim(value);
				options.length = value.length;
				options.availableLength = options.maxLength == null ? '' : options.maxLength - value.length; 
				return (options.minLength == null || value.length >= options.minLength)  && 
					(options.maxLength == null || value.length <= options.maxLength);
			}
	};
	
	var Round = Math.round;	
	var parseNumber_ = function(field, value, parser){
		// Check for number
		if(jef.isNumber(value)){
			return value;
		}
		var formatter, format;
		// Get parser
		if(parser){
			if(jef.isString(parser)){
				formatter = jef.format.getFormatter(parser);
			} else {
				formatter = {parse: parser };
			}
		} else {
			var _formats = getValidationElement_($(field), 'formats');
			for(var _f in _formats){
				if(_f.indexOf('number.') === 0){
					format = _f;
					break;
				}
			}
			if(format){
				formatter = jef.format.getFormatter(format);
			} else { 
				formatter = {parse: settings.defaultNumberParser };
			}
		}
		// Parse and check for NaN
		value = formatter.parse(value);
		if(isNaN(value))
			return NaN;
		return value;
	};
	
	var parseInteger_ = function(field, value, parser){
		value = parseNumber_(field, value, parser);
		return Round(value) !== value ? NaN : value;
	};
	
	validators['int'] = {
			MIN_VALUE : -2147483648,
			MAX_VALUE : 2147483647,
			options : {				
				min : null,
				max : null,
				invalidMessage : 'El número ingresado es inválido',
				errorMessage : 'Debe ingresar un valor incluido entre {min} y {max}.',
				parser : undefined		
			},
			parse : function(value, options){
				var value = parseInteger_(options.field, value, options.parser);
				return isNaN(value) ? null : (value >= this.MIN_VALUE && value <= this.MAX_VALUE ? value : null);
			},
			valid : function(value, options){
				return (options.min == null || value >= options.min)  && 
					(options.max == null || value <= options.max);
			}
	};	
	
	validators['long'] = {
			MIN_VALUE : -9223372036854775808,
			MAX_VALUE : 9223372036854775807,
			options : {				
				min : null,
				max : null,
				invalidMessage : 'El número ingresado es inválido',
				errorMessage : 'Debe ingresar un valor incluido entre {min} y {max}.',
				parser : undefined		
			},
			parse : function(value, options){
				var value = parseInteger_(options.field, value, options.parser);
				return isNaN(value) ? null : (value >= this.MIN_VALUE && value <= this.MAX_VALUE ? value : null);
			},
			valid : function(value, options){
				return (options.min == null || value >= options.min)  && 
					(options.max == null || value <= options.max);
			}
	};
	
	validators['short'] = {
			MIN_VALUE : -32768,
			MAX_VALUE : 32767,
			options : {				
				min : null,
				max : null,
				invalidMessage : 'El número ingresado es inválido',
				errorMessage : 'Debe ingresar un valor incluido entre {min} y {max}.',
				parser : undefined 
			},
			parse : function(value, options){
				var value = parseInteger_(options.field, value, options.parser);
				return isNaN(value) ? null : (value >= this.MIN_VALUE && value <= this.MAX_VALUE ? value : null);
			},
			valid : function(value, options){
				return (options.min == null || value >= options.min)  && 
					(options.max == null || value <= options.max);
			}
	};
	
	validators['byte'] = {
			MIN_VALUE : -128,
			MAX_VALUE : 127,
			options : {				
				min : null,
				max : null,
				invalidMessage : 'El número ingresado es inválido',
				errorMessage : 'Debe ingresar un valor incluido entre {min} y {max}.',
				parser : undefined 
			},
			parse : function(value, options){
				var value = parseInteger_(options.field, value, options.parser);
				return isNaN(value) ? null : (value >= this.MIN_VALUE && value <= this.MAX_VALUE ? value : null);
			},
			valid : function(value, options){
				return (options.min == null || value >= options.min)  && 
					(options.max == null || value <= options.max);
			}
	};
		
	validators['double'] = {
			MIN_VALUE : -1.7976931348623157E308,
			MAX_VALUE : 1.7976931348623157E308,
			options : {				
				minInclusive : null,
				maxInclusive : null,
				minExclusive : null,
				maxExclusive : null,
				invalidMessage : 'El número ingresado es inválido',
				errorMessage : 'Debe ingresar un valor incluido entre {min} y {max}.',
				parser : undefined 
			},
			parse : function(value, options){
				var value = parseNumber_(options.field, value, options.parser);
				return isNaN(value) ? null : (value >= this.MIN_VALUE && value <= this.MAX_VALUE ? value : null);
			},
			valid : function(value, options){
				var minI = options.minInclusive, minE = options.minExclusive;
				var maxI = options.maxInclusive, maxE = options.maxExclusive;
				//options.min = minI == null ? minE : minI;
				//options.max = maxI == null ? maxE : maxI;				
				return (minI == null || value >= minI) && (minE == null || value > minE) &&
					(maxI == null || value <= maxI) && (maxE == null || value < maxE);
			}
	};
	
	validators['float'] = {
			MIN_VALUE : -3.4028234663852886E38,
			MAX_VALUE : 3.4028234663852886E38,
			options : {				
				minInclusive : null,
				maxInclusive : null,
				minExclusive : null,
				maxExclusive : null,
				invalidMessage : 'El número ingresado es inválido',
				errorMessage : 'Debe ingresar un valor incluido entre {min} y {max}.',
				parser : undefined 
			},
			parse : function(value, options){
				var value = parseNumber_(options.field, value, options.parser);
				return isNaN(value) ? null : (value >= this.MIN_VALUE && value <= this.MAX_VALUE ? value : null);
			},
			valid : function(value, options){
				var minI = options.minInclusive, minE = options.minExclusive;
				var maxI = options.maxInclusive, maxE = options.maxExclusive;
				//options.min = minI == null ? minE : minI;
				//options.max = maxI == null ? maxE : maxI;				
				return (minI == null || value >= minI) && (minE == null || value > minE) &&
					(maxI == null || value <= maxI) && (maxE == null || value < maxE);
			}
	};
	
	var Date = window.Date;	
	var parseDateAtAllCost = jef.date.parseAtAllCost;
	var parseTimeAtAllCost = jef.time.parseAtAllCost;
	
	validators['date'] = {
			options : {				
				min : null,
				max : null,
				ignoreTime : true,
				ignoreMilliseconds : true,
				buildComparableDate : function(d){
					if(this.ignoreTime === true)
						return new Date(d.getFullYear(),d.getMonth(),d.getDate(), 0,0,0,0);
					if(this.ignoreMilliseconds === true)
						return new Date(d.getFullYear(),d.getMonth(),d.getDate(),
								d.getHours(),d.getMinutes(),d.getSeconds(), 0);
					return d;
				},
				invalidMessage : 'La fecha ingresada es inválida',
				errorMessage : 'Debe ingresar una fecha incluida entre {min,date} y {max,date}.' 
			},
			parse : function(value, options){
				return jef.isDateLike(value) ? value : parseDateAtAllCost(value);
			},
			valid : function(value, options){
				value = options.buildComparableDate(value);
				return (options.min == null || value >= options.buildComparableDate(options.min))  && 
					(options.max == null || value <= options.buildComparableDate(options.max));
			}
	};
	
	validators['time'] = {
			options : {				
				min : null,
				max : null,
				ignoreMilliseconds : true,
				buildComparableTime : function(d){
					if(this.ignoreMilliseconds === true)
						return new Date(2000,0,1, d.getHours(),d.getMinutes(),d.getSeconds() ,0);
					return new Date(2000,0,1, d.getHours(),d.getMinutes(),d.getSeconds(),d.getMilliseconds());
				},
				invalidMessage : 'La hora ingresada es inválida',
				errorMessage : 'Debe ingresar una hora incluida entre {min,time} y {max,time}.' 
			},
			parse : function(value, options){
				return jef.isDateLike(value) ? value : parseTimeAtAllCost(value);
			},
			valid : function(value, options){
				value = options.buildComparableTime(value);
				return (options.min == null || value >= options.buildComparableTime(options.min))  && 
					(options.max == null || value <= options.buildComparableTime(options.max));
			}
	};
	
	validators['email'] = {
			options : {
				regex : /\b(^[_A-Za-z0-9-]+(\.[_A-Za-z0-9-]+)*@([A-Za-z0-9-])+(\.[A-Za-z0-9-]+)*((\.[A-Za-z0-9]{2,})|(\.[A-Za-z0-9]{2,}\.[A-Za-z0-9]{2,}))$)\b/gi,
				errorMessage : 'El e-mail es inválido.' 
			},
			valid : function(value, options){	
				return value.length == 0 || value.match(options.regex) != null;
			}
	};
	
	validators['regex'] = {
			options : {
				regex : /(.*)/,
				errorMessage : 'El campo no cumple con la expressión regular' 
			},
			valid : function(value, options){	
				return value.length == 0 || value.match(options.regex) != null;
			}
	};
	
	validators['expression'] = {
			options : {
				expression : "true",
				errorMessage : 'El campo no cumple con la expressión' 
			},
			valid : function(value, options){	
				return value.length == 0 || valid.evalExpression(options.expression, options);
			}
	};
	
	validators['url'] = {
			options : {
				regex : /(^(ftp|http|https):\/\/(\.[_A-Za-z0-9-]+)*(@?([A-Za-z0-9-])+)?(\.[A-Za-z0-9-]+)*((\.[A-Za-z0-9]{2,})|(\.[A-Za-z0-9]{2,}\.[A-Za-z0-9]{2,}))(:[0-9]+)?([\/A-Za-z0-9?#_-]*)?$)/gi,
				errorMessage : 'La url es inválida.' 
			},
			valid : function(value, options){	
				return value.length == 0 || value.match(options.regex) != null;
			}
	};
	
	var Rut = jef.util.Rut;
	
	validators['rut'] = {
			options : {
				dot : true,
				dvField : null,
				errorMessage : 'El RUT es inválido.' 
			},
			valid : function(value, options){
				var dv;
				if( options.dvField ){
					dv = wrappFieldName_(options.dvField, options.formId).val();
					if( !dv ){
						return true;
					}
					value = value + '-' + dv;
				}
				return value.length == 0 || new Rut(value, options.dot).valid;
			}
	};
		
	/********************************************************************/
	/**************           Built-in Features            **************/
	/********************************************************************/
		
	var keyFeatures = (function(jef, settings){		
		/** Parse key block expression to regex */	
		var parseKeyBlockRegex_ = function(r){
			if( jef.isString(r) ){
				return new RegExp('[' + r + ']');
			} else if ( r instanceof RegExp ){
				return r;
			}
			log.warn('[jef.valid.parseKeyBlock] uncorrect key block expression, must be ' + 
				'a string or a regular expression : ' + r);
		};
		/** Test if a charCode must be skipped during keyBlock validation */			
		var isSkipCharCode_ = function(charCode){
			var charCodes = settings.keyBlockSkippedCharCode;
			for(var i=charCodes.length; i--;){
				if( charCodes[i] === charCode ){
					return true;
				}
			}
			return false;
		};
		/** keypress event */
		var keypress_ = function(options, event){
			var charCode = event.which;
			if( event.ctrlKey || event.altKey || event.metaKey || 
				isSkipCharCode_(charCode, event) ) {
				return true;
			}
			var s = String.fromCharCode(charCode), d = options.rdeny, a = options.rallow;
			return (!d || !d.test(s)) && (!a || a.test(s));
		};
		return [{			
				parseOptions : function(options){
					var rdeny, rallow;
					if( jef.isSimpleObject(options) ){			
						rallow = options.allow && parseKeyBlockRegex_(options.allow); 
						rdeny = options.deny && parseKeyBlockRegex_(options.deny);
					} else if ( !(rdeny = parseKeyBlockRegex_(options)) ) {
						log.warn('[jef.valid.feature.keyBlock] id = ' + we[prop]('id') + ', uncorrect key block value : ' + value);
					}
					return { rdeny: rdeny, rallow: rallow};
				}, keypress : keypress_
			} , { 
				parseOptions: function(options){
					var rdeny, rallow;
					if( jef.isSimpleObject(options) ){			
						rallow = options.allow && parseKeyBlockRegex_(options.allow); 
						rdeny = options.deny && parseKeyBlockRegex_(options.deny);
					} else if ( !(rallow = parseKeyBlockRegex_(options)) ) {
						log.warn('[jef.valid.feature.keyPass] id = ' + we[prop]('id') + ', uncorrect key block value : ' + value);
					}
					return { rdeny: rdeny, rallow: rallow};
				}, keypress: keypress_
			}];
	})(jef, settings);
	
	// keyBlock : enable/disable keys entered by user	
	features['keyBlock'] = keyFeatures[0];	
	// keyPass : same as keyBlock but allow by default
	features['keyPass'] = keyFeatures[1];
	
	// pasteBlock : disabled user to paste something in input
	features['pasteBlock'] = {
		paste : function(options, events) { return !options; }
	};
	
	features['emptyText'] = {
		parseOptions : function(options){
			var text, css;
			if(jef.isString(options)){
				text = options;
				css = settings.defaultEmptyTextStyle;
			} else {
				text = '' + options.text;
				css = options.css === undefined ? settings.defaultEmptyTextStyle : options.css;
				for(var property in options){
					if(property == 'text' || property == 'css')
						continue;
					css[property] = options[property];
				}
			}
			return { text: text, css: css };
		},
		initField : function(options, field){
			var value = field.val();
			// Check for maxlength
			var maxlength = field.attr('maxlength');
			var emptyTextLength = options.text.length;
			if(maxlength && maxlength > 0 && emptyTextLength > maxlength){
				options.text = options.text.substr(0, maxlength);
				jef.log.error(jef.tpl('[{0}] Empty text length ({1}) is greater than maxlength ({2}) attribute.',
						[field[prop]('id'), emptyTextLength, maxlength]));
			}
			if(options.css){
				var originalCss = {}, key, value;
				for(key in options.css){
					originalCss[key] = field.css(key);
				}
				options.originalCss = originalCss;
			}
			if(value === ''){
				field.val(options.text);
				if(options.css){
					field.css(options.css);
				}
			}
		},
		finalizeField : function(options, field){
			var value = field.val();
			if(value === options.text){
				field.val('');
				if(options.originalCss){
					field.css(options.originalCss);
				}
			}
		},
		validationValue : function(options, value){
			if(options.text === value)
				return '';
			return value;
		},
		focus : function(options, event){
			var we = $(this), val = we.val();
			if(val === options.text){
				we.val('');
				if(options.originalCss){
					we.css(options.originalCss);
				}
			}
			options.out = false;
		},
		blur : function(options, event){
			var we = $(this), val = we.val();
			if(val === '' || val === options.text){
				we.val(options.text);
				if(options.css){
					we.css(options.css);
				}
			}
			options.out = true;
		},
		change : function(options, event){
			if(options.out !== true) 
				return;
			var we = $(this), val = we.val();
			if(val === options.text){
				if(options.css){
					we.css(options.css);
				}
			} else if(val !== ''){
				if(options.originalCss){
					we.css(options.originalCss);
				}
			}
		}
	};
	
	features['datepicker'] = {
		parseOptions : function(options){
			return options || {};
		},
		initField : function(options, field){				
			// Format
			if( options.dateFormat == null ){
				var formats = getValidationElement_(field, 'formats'), format;
				for(format in formats){
					if( format.indexOf('date') === 0 ){
						break;
					}
					format = null;
				}
				format = format || 'date';				
				options.dateFormat = jef.format.getPattern(format);
			}
			// Trigger
			if( options.trigger != null ){
				options.showOn = options.showOn || 'none',
				$(options.trigger).bind('click.validationFeatureDatepicker', function(event){
					field.datepicker('show');
					return false;
				});
			}
			// On Close event
			options.onClose = function(text, dtp){
				field.validate();
			};
			// Create datetimepicker
			field.datepicker(options);			
		},
		finalizeField : function(options, field){
			field.datepicker( "destroy" );
			if(options.trigger != null){
				$(options.trigger).unbind('click.validationFeatureDatepicker');
			}
		}
	};
		
	/********************************************************************/
	/**************           Built-in Outputs             **************/
	/********************************************************************/
	
	valid.outputs.field('noop', outputs.noop);
	valid.outputs.field('span', {
		options : {
			errorId : function(e){
				return 'span_v_error_'+ underscoreNW_(e.name);
			},
			errorClass : '',
			successId : function(e){
				return 'span_v_success_'+ underscoreNW_(e.name);
			},
			successClass : ''
		},
		addErrorMessage : function(message, domElement){
			var spanId = this.options.errorId(domElement);
			var span = $('#' + spanId);
			if(span.size() == 0){				
				span = $('<span id="'+spanId+'" class="'+this.options.errorClass+'" ></span>');				
				$(domElement).parent().append(span);
			} else {				
				span.show();
			}	
			span.html(message);
		},
		addSuccessMessage : function(message, domElement){
			var spanId = this.options.successId(domElement);
			var span = $('#' + spanId);
			if(span.size() == 0){
				span = $('<span id="'+spanId+'" class="'+this.options.successClass+'"></span>');
				$(domElement).parent().append(span);
			} else {
				span.show();
			}
			span.html(message);
		},
		clearErrorMessage : function(domElement){	
			$('#' + this.options.errorId(domElement)).hide();
		},
		clearSuccessMessage : function(domElement){	
			$('#' + this.options.successId(domElement)).hide();
		}
	});
	
	valid.outputs.form('noop', outputs.noop);	
	valid.outputs.form('ul', {
		options : {
			targetId : function(formId){
				return formId + '_validation';
			},
			fieldItemId : function(formId, fieldName){
				return 'li_validation_' + formId + '_' + underscoreNW_(fieldName);
			},
			targetClass : '',
			itemClass : '',
			anchor : true
		},
		getTarget : function(formId){
			var targetId = this.options.targetId(formId);
			var target = $('#' + targetId);
			if(target.size() < 1){
				target = $('<ul id="'+targetId+'" class="'+this.options.targetClass+'"></ul>');
				target.insertBefore($('#' + formId));
			}	
			target.formId = formId;
			return target;
		},
		onValidationStart : function(target){
			target.empty().hide();
		},
		addErrorMessage : function(target, message, domElement) {			
			var opt = this.options, eName = domElement && domElement.name, li, lid, formId = target.formId;
			if( opt.anchor && eName ){				
				message = $('<a href="#">'+ message +'</a>');
				message.click(function(event){
					domElement && domElement.focus();
					return false;
				});
			}
			if( eName ) {
				lid = this.options.fieldItemId(formId, eName);
				li = $('#' + lid);
				if ( li.size() === 0 ){
					li = $('<li class="'+opt.itemClass+'" id="' + lid + '" ></li>');
					target.append(li);
				} else {
					li.children().remove();
				}
			} else {
				li = $('<li class="'+opt.itemClass+'" ></li>');
				target.append(li);
			}
			li.append(message);
		},
		addSuccessMessage : function(target, message, domElement){
			// noop
		},
		clearErrorMessage : function(target, domElement){	
			if( domElement ){
				var lid = this.options.fieldItemId(target.formId, domElement.name);
				$('#' + lid).remove();
			}
		},
		clearSuccessMessage : function(target, domElement){	
			// noop
		},
		onValidationFail : function(target, options){
			var a, id = target.show()[prop]('id');
			if(this.options.anchor && (a = $('#' + id + ' a ')[0]) ){
				focus_($(a));
			} else if(!target.is(":empty")){
				goTo_(target);
			} else {
				target.hide();
			}
		},
		onValidationSuccess : function(target, options){
			// noop
		}
	});
	
	
	valid.outputs.form('field', {
		options : {
			formOutput: outputs.forms.ul
		},		
		getTarget : function(formId) { 
			var target = {
				formId: formId
			};
			if(this.options.formOutput){
				target = this.options.formOutput.getTarget(formId);  
				target.formId = formId;
			}
			// Set field output			
			var _v = getValidationData_($('#' + formId));
			target.fieldOutput = (_v.formOutputs && _v.formOutputs.field) || outputs.field;
			return target;
		},
		onValidationStart : function(target){
			var formId = target.formId;			
			var output = target.fieldOutput;
			$('#' + formId + ' :field').each(function(i, e){
				output.clearErrorMessage(e);
				output.clearSuccessMessage(e);
			});			
			if(this.options.formOutput){
				this.options.formOutput.onValidationStart(target);
			}
		},
		addErrorMessage : function(target, message, domElement) {			
			if(domElement){
				target.fieldOutput.addErrorMessage(message, domElement);
			} else if(this.options.formOutput){
				this.options.formOutput.addErrorMessage(target, message);
			}
		},
		addSuccessMessage : function(target, message, domElement){
			if(domElement){
				target.fieldOutput.addSuccessMessage(message, domElement);
			} else if(this.options.formOutput){
				this.options.formOutput.addSuccessMessage(target, message);
			}
		},
		clearErrorMessage : function(target, domElement){	
			if(domElement){
				target.fieldOutput.clearErrorMessage(domElement);
			} else if(this.options.formOutput){
				this.options.formOutput.clearErrorMessage(target);
			}
		},
		clearSuccessMessage : function(target, domElement){	
			if(domElement){
				target.fieldOutput.clearSuccessMessage(domElement);
			} else if(this.options.formOutput){
				this.options.formOutput.clearSuccessMessage(target);
			}
		},
		onValidationFail : function(target, options){
			var formId = target.formId;
			var i, err, errs = options.errorData;
			for(i in errs){
				err = errs[i];
				if(err.fieldName){
					focus_(wrappFieldName_(err.fieldName, formId));
					return;
				}
			}
			if(this.options.formOutput){
				this.options.formOutput.onValidationFail(target, options);
			}
		},
		onValidationSuccess : function(target, options){
			if(this.options.formOutput){
				this.options.formOutput.onValidationSuccess(target, options);
			}
		}
	});
	
	valid.outputs('span', 'ul');
		
	/********************************************************************/
	/**************           Expression Parser            **************/
	/********************************************************************/
	
	/** Place holders container */	
	var expressionPlaceholders = {};
	/** Replace place holders */
	var replaceExpressionPlaceholders_ = function(text){
	
		var index = 0, length = text.length;
		var result = '';
		var open, rule, nextChar, params, endIndex;
		while(index < length){						
			// Search dollard			
			var dollardIndex = text.indexOf('$', index);			
			// No dollard found, flush to end
			if(dollardIndex == -1){
				result += text.substring(index, length);
				return result;
			}			
			// Flush to dollardIndex
			result += text.substring(index, dollardIndex);
			// Go next
			index = dollardIndex + 1;
			if(index >= length)
				return result + '$';
			// Look for rules
			rule = null;
			for(open in expressionPlaceholders){
				if(text.indexOf(open, index) === index){
					rule = expressionPlaceholders[open];
					index += open.length - 1;
					break;
				}
			}			
			// Replace
			if(rule){
				// Go next
				index ++;
				if(index >= length)
					return result + '$' + open;
				// Seach end of rule
				endIndex = text.indexOf(rule[0], index);
				if(endIndex == -1){
					result += '$' + open;
					continue;
				}
				if(endIndex == index){
					result += '$' + open + rule[0];
					index++;
					continue;
				}
				// Get params (comma-separated), call format method and flush to result
				params = text.substring(index, endIndex).split(',');
				result += rule[1].call(rule[2], params); 				
				index = endIndex + 1;
				continue;
			}			
			result += '$' + text.charAt(index);
			index++;
		}
		
		return result;
	};
	/** Parse expression */
	var parseExpression_ = function(expression){
		if(jef.isFunction(expression)){
			return expression;
		} else
		if(!jef.isString(expression)){
			return null;
		}
		// Start with ? -> check for not empty value
		if(expression.indexOf('?') === 0){
			expression = expression.substr(1).split(',');
			return function(){
				for(var i=0; i<expression.length; i++){
					if(valid.getFieldValue(expression[i]) === '')
						return false;
				}
				return true;
			};
		// Start with ! -> check for empty value
		} else if(expression.indexOf('!') === 0){
			expression = expression.substr(1).split(',');
			return function(){
				for(var i=0; i<expression.length; i++){
					if(valid.getFieldValue(expression[i]) !== '')
						return false;
				}
				return true;
			};
		}
		// Replace place holders
		expression = replaceExpressionPlaceholders_(expression);
		// Return EVAL function
		var format = jef.msg.format;
		return function(){
			return eval(format(expression, this, false));
		};
	};
	
	valid.addExpressionPlaceholder = function(open, close, replace){
		if(jef.isString(replace)){
			var format = jef.msg.format;
			replace = function(params){
				return format(replace, params, false);
			};
		}
		expressionPlaceholders[open] = [close, replace, {open : open, close : close}];
	};
	
	// Built-in placeHolders
	valid.addExpressionPlaceholder('{', '}', function(params) {
		if(params.length <= 1){
			return "jef.valid.getFieldValue('" + params[0] + "')";
		} else {
			return "jef.valid.getFieldValue('" + params[0] + "','" + params[1] + "')";
		}
	});
	valid.addExpressionPlaceholder('[', ']', function(params) {
		if(params.length <= 1){
			return "jef.valid.getFieldValues('" + params[0] + "')";
		} else {
			return "jef.valid.getFieldValues('" + params[0] + "','" + params[1] + "')";
		}
	});
	valid.addExpressionPlaceholder('V{', '}', function(params) {
		if(params.length <= 1){
			return "jef.valid.isFieldValid('" + params[0] + "')";
		} else {
			return "jef.valid.isFieldValid('" + params[0] + "','" + params[1] + "')";
		}
	});
		
	valid.parseExpression = parseExpression_;
	valid.evalExpression = function(expression, context){
		return parseExpression_(expression).call(context || {});
	};
		
	/********************************************************************/
	/**************                Utils                   **************/
	/********************************************************************/
	
	// Shorcut by local variables
	var log = jef.log;
	var trim = jef.string.trim;
	var isIn = jef.string.isIn;
	var isFct = jef.isFunction;
	var sEquals = jef.string.equals;
	var aconcat = jef.array.concat;
	var acopy = jef.array.copy;
	var aremove = jef.array.remove;
	
	/**
	* Determine if a field (input, ..) is multiple.
	*/
	var isMultiple_ = function(we){
		return isIn('' + we.attr('type'), ['checkbox', 'radio']);
	};	
	/**
	* Get format key from class name
	*/
	var formatKeyFromClass_ = function(className){
		return className.substr(2).replace('_','.');
	};
	/**
	* Transfrom an array or a space-separated string list, to an array of lower-case strings.
	*/	
	var fixSplitArray_ = function(array, isString){
		if(isString === true || jef.isString(array)){
			var split = array.split(' '), e;
			array = [];
			for(var i=0; i<split.length; i++){
				e = split[i];
				if( e.length === 0 )
					continue;
				array.push(e.toLowerCase());
			}
		} else {
			for(var i=0, l=array.length; i<l; i++){
				array[i] = array[i].toLowerCase();
			}
		}
		return array;
	};
	/** Blur the active element and then focus */
	var focus_ = function(we){		
		// Fix hidden
		if(!we.is(':focusable')){
			goTo_(we);
		} else {
			blurActiveElement_();
			we.focus();
		}
	};	
	
	/** Blur active element on document */
	var blurActiveElement_ = function(){
		var activeElement = document.activeElement;	
		if(activeElement){
			$(activeElement).blur();
		}
		return activeElement;
	};
	
	/** Focus appending a temporal anchor after (or before) this element */
	var goTo_ = function(target){
		var activeElement = blurActiveElement_();
		var activeTop = activeElement && $(activeElement).offset().top;
		var top = !isNaN(activeTop) && target.offset().top;		
		var tmp = $('<a href="#">&nbsp;</a>');
		if(!isNaN(top) && top > activeTop) { 
			tmp.insertAfter(target); 
		} else { 
			tmp.insertBefore(target); 			
		}
		tmp.focus();
		tmp.remove();	
	};
	
	var rEscapeNW = /(\W)/g;	
	var escapeNW_ = function(txt){
		return txt.replace(rEscapeNW, '\\$1');
	};
	
	var rUnderscoreNW = /\W/g;
	var underscoreNW_ = function(txt){
		return txt.replace(rUnderscoreNW, '_');
	};
	
	var max_ = function(i, j) {
		return i>j ? i : j;
	};
	
	var min_ = function(i, j) {
		return i>j ? j : i;
	};
	
	// Field Value
		
	/**
	* Create a wrap element from name
	*/
		
	var wrappFieldName_ = function(name, formId){
		var we;
		if(formId){
			we = $('#' + formId + ' :field').filter(function(i){
			    return this.name === name;
			});
		} else {
			we = $('form :field').filter(function(i){
			    return this.name === name;
			});
		}
		return we;
	};
	/**
	* Get value of a field, fixing validation value (ie: emptyText). If isMultiple (radio, checkbox), it will return first value or ''.
	*/
	var getValidationFieldValue_ = function(we, _v, _validation){	
		_v = _v || getValidationData_(we);
		_validation = _validation || buildValidation_(we, _v);
		var value = getFieldValue_(we, _v);
		if(_validation.validationValue){
			value = _validation.validationValue(value);
		}
		return value;
	};
	/**
	* Get value of a field. If isMultiple (radio, checkbox), it will return first value or ''.
	*/
	var getFieldValue_ = function(we, _v){		
		_v = _v || getValidationData_(we);
		var val;
		if(_v.isMultiple){					
			val = $( _v.getFieldValueSelector ).val();
		} else {
			val = we.val();			
		}
		return jef.isArray(val) ? (val.length === 0 ? '' : val[0]) : (val == null ? '' : val);
	};
	/**
	* Get all values of a field.
	*/
	var getFieldValues_ = function(we, _v){
		_v = _v || getValidationData_(we);
		var fieldsValue = [], val;
		$( _v.getFieldValueSelector ).each(function(i,e){
			aconcat(fieldsValue, $(e).val() || '');
		});
		return fieldsValue;
	};
	/**
	* Set value of a field. If isMultiple (radio, checkbox), it will be set as array.
	*/
	var setFieldValue_ = function(we, value, _v) {
		_v = _v || getValidationData_(we);
		var avalue = jef.isArray(value) ? value : [value];
		if(_v.isMultiple){			
			$( _v.setFieldValueSelector ).val(avalue);
		} else {
			we.val(value);
		}
		return avalue;
	};
	
	valid.getFieldValue = function(name, formId){
		var we = wrappFieldName_(name, formId);
		return getValidationFieldValue_(we);
	};	
	valid.getFieldValues = function(name, formId){
		var we = wrappFieldName_(name, formId);
		return getFieldValues_(we);
	};
	valid.setFieldValue = function(name, formId, value){
		if(value === undefined){
			value = formId;
			formId = null;
		}
		var we = wrappFieldName_(name, formId);
		return setFieldValue_(we, value);
	};
	
	// Check element	
	/**
	* Check for DOM element tag name
	*/
	var checkTag_ = function(we, tagNames, fctName){
		var domElement = we[0];
		if(!isIn(domElement.tagName,tagNames,true))
		{
			log.error('['+fctName+'] domElement tag must be one of [' + tagNames + 
				'] | id = ' + domElement.id);
			return false;
		}
		return true;
	};
	/**
	* Check for DOM element tag name and type
	*/	
	var checkTagsAndType_ = function(we, tagNames, type, fctName){
		var domElement = we[0];
		if(!isIn(domElement.tagName,tagNames,true) && 
			(!domElement.type || !sEquals(domElement.type,type,true)))
		{
			log.error('['+fctName+'] domElement tag must be one of ['+ tagNames +
					'], and must have type='+type+' | id = ' + domElement.id);
			return false;
		}	
		return true;
	};
		
})(jef, jQuery);

